@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """edX Enterprise's Consent application.
 
 The consent application provides an API that enables the platform to prompt
@@ -13,5 +12,3 @@ of gate that an enterprise stands at.
 """
 
 __version__ = "0.1.0"
-
-default_app_config = "consent.apps.ConsentConfig"  # pylint: disable=invalid-name
