@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Django admin integration for the Consent application.
 """
@@ -10,7 +9,7 @@ from django.contrib import admin
 from django.http import HttpResponseRedirect
 from django.urls import reverse
 from django.utils.http import urlencode
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from consent.models import DataSharingConsent, DataSharingConsentTextOverrides
 
@@ -95,26 +94,30 @@ class DataSharingConsentTextOverridesAdmin(DjangoObjectActions, SimpleHistoryAdm
         url = reverse('grant_data_sharing_permissions') + '?{}'.format(urlencode(params))
         return HttpResponseRedirect(url)
 
-    def preview_as_course(self, request, consent_page):  # pylint: disable=unused-argument
+    @admin.action(
+        description=_(
+            "Preview the data sharing consent page rendered in the context of a course enrollment."
+        )
+    )
+    def preview_as_course(self, request, consent_page):
         """
         Renders data sharing consent page in course context
         """
         return self.preview(consent_page, course_id='course-v1:edX+TestX+Test_Course')
 
-    def preview_as_program(self, request, consent_page):  # pylint: disable=unused-argument
+    @admin.action(
+        description=_(
+            "Preview the data sharing consent page rendered in the context of a program enrollment."
+        )
+    )
+    def preview_as_program(self, request, consent_page):
         """
         Renders data sharing consent page in program context
         """
         return self.preview(consent_page, program_uuid='25c10a26-0b00-0000-bd06-7813546c29eb')
 
     preview_as_course.label = _("Preview (course)")
-    preview_as_course.short_description = _(
-        "Preview the data sharing consent page rendered in the context of a course enrollment."
-    )
     preview_as_program.label = _("Preview (program)")
-    preview_as_program.short_description = _(
-        "Preview the data sharing consent page rendered in the context of a program enrollment."
-    )
 
     class Meta:
         """
