@@ -1,11 +1,9 @@
-# -*- coding: utf-8 -*-
 """
 Mixins for edX Enterprise's Consent application.
 """
 import logging
 
 from django.contrib import auth
-from django.utils.encoding import python_2_unicode_compatible
 
 from enterprise.models import EnterpriseCourseEnrollment
 
@@ -13,7 +11,6 @@ LOGGER = logging.getLogger(__name__)
 User = auth.get_user_model()
 
 
-@python_2_unicode_compatible
 class ConsentModelMixin:
     """
     A mixin for Data Sharing Consent classes that require common, reusable functionality.
@@ -41,7 +38,7 @@ class ConsentModelMixin:
         """
         children = getattr(self, '_child_consents', [])
         if children:
-            return any((child.consent_required() for child in children))
+            return any(child.consent_required() for child in children)
 
         if self.granted:
             return False
