@@ -1,26 +1,30 @@
-# -*- coding: utf-8 -*-
 """
 Forms to be used in the enterprise djangoapp.
 """
 
 import re
 from logging import getLogger
+from uuid import UUID
 
+from config_models.admin import ConfigurationModelAdmin
 from edx_rbac.admin.forms import UserRoleAssignmentAdminForm
 
 from django import forms
 from django.conf import settings
-from django.contrib import auth
+from django.contrib import admin, auth
 from django.core.exceptions import ValidationError
+from django.db.models import Q
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.urls import reverse
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from enterprise import utils
 from enterprise.admin.utils import email_or_username__to__email, split_usernames_and_emails
 from enterprise.admin.widgets import SubmitInput
 from enterprise.models import (
+    AdminNotification,
+    BulkCatalogQueryUpdateCommandConfiguration,
     EnterpriseCustomer,
     EnterpriseCustomerCatalog,
     EnterpriseCustomerIdentityProvider,
@@ -35,7 +39,7 @@ try:
 except ImportError:
     saml_provider_configuration = None
 
-logger = getLogger(__name__)  # pylint: disable=invalid-name
+logger = getLogger(__name__)
 User = auth.get_user_model()
 
 
@@ -73,6 +77,7 @@ class ManageLearnersForm(forms.Form):
             ("no-id-professional", _("Professional Education (no ID)")),
             ("credit", _("Credit")),
             ("honor", _("Honor")),
+            ("unpaid-executive-education", _("Unpaid Executive Education")),
         ],
     )
     reason = forms.CharField(label=_("Reason for manual enrollment"), required=False)
@@ -379,6 +384,7 @@ class EnterpriseCustomerAdminForm(forms.ModelForm):
         fields = (
             "name",
             "slug",
+            "auth_org_id",
             "country",
             "active",
             "customer_type",
@@ -389,19 +395,33 @@ class EnterpriseCustomerAdminForm(forms.ModelForm):
             "enable_audit_data_reporting",
             "replace_sensitive_sso_username",
             "hide_course_original_price",
+            "disable_expiry_messaging_for_learner_credit",
             "enable_portal_code_management_screen",
             "enable_portal_subscription_management_screen",
             "enable_learner_portal",
+            "enable_learner_portal_offers",
+            "enable_portal_learner_credit_management_screen",
+            "enable_executive_education_2U_fulfillment",
+            "hide_labor_market_data",
             "enable_integrated_customer_learner_portal_search",
+            "enable_learner_portal_sidebar_message",
+            "learner_portal_sidebar_content",
+            "enable_pathways",
+            "enable_programs",
+            "enable_demo_data_for_analytics_and_lpr",
             "enable_analytics_screen",
             "enable_portal_reporting_config_screen",
             "enable_portal_saml_configuration_screen",
             "enable_portal_lms_configurations_screen",
+            "enable_universal_link",
+            "enable_browse_and_request",
             "enable_slug_login",
+            "show_videos_in_learner_portal_search_results",
             "contact_email",
             "default_contract_discount",
             "default_language",
             "sender_alias",
+            "reply_to",
         )
 
 
@@ -417,7 +437,7 @@ class EnterpriseCustomerCatalogAdminForm(forms.ModelForm):
                                  help_text=_("Hold Ctrl when clicking on button to open Preview in new tab"))
 
     @staticmethod
-    def get_catalog_preview_uuid(post_data):  # pylint: disable=invalid-name
+    def get_catalog_preview_uuid(post_data):
         """
         Return the uuid of the catalog the preview button was clicked on
         There must be only one preview button in the POST data.
@@ -558,6 +578,7 @@ class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
             "data_type",
             "report_type",
             "delivery_method",
+            "enable_compression",
             "pgp_encryption_key",
             "frequency",
             "day_of_month",
@@ -586,11 +607,11 @@ class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
         report_customer = cleaned_data.get('enterprise_customer')
         data_type = cleaned_data.get('data_type')
 
-        if data_type in EnterpriseCustomerReportingConfiguration.PEARSON_ONLY_REPORTS \
-                and report_customer.name != 'Pearson':
+        if data_type in EnterpriseCustomerReportingConfiguration.MANUAL_REPORTS \
+                and report_customer.uuid not in map(UUID, settings.ENTERPRISE_MANUAL_REPORTING_CUSTOMER_UUIDS):
             message = _(
-                'This data_type "{data_type}" is not supported for enterprise'
-                'customer {enterprise_customer}. Please select a different data_type.',
+                '"{data_type}" data type is not supported for enterprise '
+                'customer "{enterprise_customer.name}". Please select a different data type.',
             ).format(
                 enterprise_customer=report_customer,
                 data_type=data_type,
@@ -614,6 +635,16 @@ class EnterpriseCustomerReportingConfigAdminForm(forms.ModelForm):
             )
             self.add_error('enterprise_customer_catalogs', message)
 
+        # Check delivery_method validity while updating report
+        create_report = self.instance.pk is not None
+        if create_report:
+            error = (self.instance.validate_delivery_method(
+                self.instance.uuid,
+                cleaned_data.get('delivery_method')
+            ))
+            if error:
+                self.add_error('delivery_method', error.get('delivery_method'))
+
 
 class TransmitEnterpriseCoursesForm(forms.Form):
     """
@@ -673,6 +704,30 @@ class SystemWideEnterpriseUserRoleAssignmentForm(UserRoleAssignmentAdminForm):
         except SystemWideEnterpriseUserRoleAssignment.user.RelatedObjectDoesNotExist:
             self.fields['enterprise_customer'].queryset = EnterpriseCustomer.objects.none()
 
+    def clean(self):
+        """
+        Ensure that the form's enterprise customer value defaults to the first active, linked enterprise customer
+        by the provided user
+        """
+        cleaned_data = super().clean()
+        user = cleaned_data.get('user')
+        enterprise_customer = cleaned_data.get('enterprise_customer')
+        applies_to_all_contexts = cleaned_data.get('applies_to_all_contexts', False)
+        if not enterprise_customer and not applies_to_all_contexts:
+            default_ent_customer = EnterpriseCustomer.objects.filter(
+                enterprise_customer_users__user_id=user.id, active=True
+            ).first()
+
+            if not default_ent_customer:
+                message = _(
+                    'User specified is not linked to an active Enterprise Customer. '
+                    'Select `Applies to all contexts` or choose a different user.',
+                )
+                logger.exception(message)
+                raise ValidationError(message)
+            cleaned_data['enterprise_customer'] = default_ent_customer
+        return cleaned_data
+
 
 class EnterpriseFeatureUserRoleAssignmentForm(UserRoleAssignmentAdminForm):
     """
@@ -682,3 +737,59 @@ class EnterpriseFeatureUserRoleAssignmentForm(UserRoleAssignmentAdminForm):
     class Meta:
         model = EnterpriseFeatureUserRoleAssignment
         fields = ['user', 'role']
+
+
+class AdminNotificationForm(forms.ModelForm):
+    """
+    Alternate form for AdminNotification.
+    """
+
+    class Meta:
+        model = AdminNotification
+        fields = '__all__'
+        widgets = {
+            'text': forms.Textarea(attrs={'cols': 80, 'rows': 5}),
+        }
+
+    def clean(self):
+        """
+        1. `start_date` and `expiration_date` are mandatory.
+        2. `start_date` must always come before the `expiration_date`.
+        3.  There must be only one admin notification in a date range.
+        """
+        cleaned_data = super().clean()
+
+        start_date = cleaned_data.get('start_date', None)
+        expiration_date = cleaned_data.get('expiration_date', None)
+
+        if start_date is None or expiration_date is None:
+            # Start and expiration dates are mandatory.
+            raise ValidationError('Start and expiration dates are mandatory.')
+
+        if expiration_date < start_date:
+            # `start_date` must always come before the `expiration_date`
+            raise ValidationError({'expiration_date': ['Expiration date should be after start date.']})
+        admin_notification = AdminNotification.objects.filter(
+            Q(start_date__range=(start_date, expiration_date)) |
+            Q(expiration_date__range=(start_date, expiration_date)) |
+            Q(start_date__lt=start_date, expiration_date__gt=expiration_date)
+        ).exclude(
+            pk=self.instance.id
+        ).exists()
+
+        if admin_notification:
+            # This should not happen, as there can be only one admin notification instance in a particular date range.
+            message = _(
+                'Please select different date range. There is another notification scheduled in this date range.',
+            )
+            logger.exception(message)
+
+            raise ValidationError(message)
+        return cleaned_data
+
+
+@admin.register(BulkCatalogQueryUpdateCommandConfiguration)
+class BulkCatalogQueryUpdateCommandConfigurationAdmin(ConfigurationModelAdmin):
+    """
+    Admin form for the BulkCatalogQueryUpdateCommandConfiguration model
+    """
