@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Custom Django Admin views used in enterprise app.
 """
@@ -6,17 +5,21 @@ Custom Django Admin views used in enterprise app.
 import datetime
 import logging
 
+from rest_framework.response import Response
+from rest_framework.views import APIView
+
 from django.conf import settings
 from django.contrib import admin, auth, messages
 from django.contrib.auth import get_permission_codename
 from django.core.exceptions import ValidationError
 from django.core.management import call_command
+from django.db import transaction
 from django.db.models import Q
-from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
+from django.http import HttpResponse, HttpResponseForbidden, HttpResponseRedirect, HttpResponseServerError, JsonResponse
 from django.shortcuts import get_object_or_404, render
 from django.urls import reverse
-from django.utils.translation import ugettext as _
-from django.utils.translation import ungettext
+from django.utils.translation import gettext as _
+from django.utils.translation import ngettext
 from django.views.generic import View
 
 from enterprise.admin.forms import (
@@ -31,10 +34,14 @@ from enterprise.admin.utils import (
     parse_csv,
     split_usernames_and_emails,
 )
+from enterprise.api_client.discovery import get_course_catalog_api_service_client
 from enterprise.api_client.ecommerce import EcommerceApiClient
-from enterprise.constants import PAGE_SIZE
+from enterprise.api_client.sso_orchestrator import EnterpriseSSOOrchestratorApiClient, SsoOrchestratorClientError
+from enterprise.constants import DJANGO_ADMIN_MANAGE_LEARNERS_LIMIT, PAGE_SIZE
+from enterprise.errors import LinkUserToEnterpriseError
 from enterprise.models import (
     EnrollmentNotificationEmailTemplate,
+    EnterpriseCatalogQuery,
     EnterpriseCustomer,
     EnterpriseCustomerUser,
     PendingEnterpriseCustomerUser,
@@ -44,6 +51,7 @@ from enterprise.utils import (
     delete_data_sharing_consent,
     enroll_users_in_course,
     get_ecommerce_worker_user,
+    get_sso_orchestrator_configure_edx_oauth_path,
     validate_course_exists_for_enterprise,
     validate_email_to_link,
 )
@@ -55,6 +63,8 @@ except ImportError:
     create_manual_enrollment_audit = None
 User = auth.get_user_model()
 
+LOG = logging.getLogger(__name__)
+
 
 class TemplatePreviewView(View):
     """
@@ -101,6 +111,30 @@ class TemplatePreviewView(View):
         return request.user.first_name or request.user.username
 
 
+class CatalogQueryPreviewView(APIView):
+    """
+    Renders a search/all response from Discovery for a particular catalog query.
+    """
+
+    def get(self, request, catalog_query_id):
+        """
+        Render the response for a particular catalog query.
+        """
+        enterprise_catalog_query = get_object_or_404(EnterpriseCatalogQuery, pk=catalog_query_id)
+        return Response(self.get_response_from_discovery(request, enterprise_catalog_query))
+
+    @staticmethod
+    def get_response_from_discovery(request, catalog_query):
+        """
+        POST a query to Discovery's search/all endpoint and return the results.
+        """
+        response = get_course_catalog_api_service_client().get_catalog_results(
+            catalog_query.content_filter,
+            traverse_pagination=True
+        )
+        return response
+
+
 class BaseEnterpriseCustomerView(View):
     """
     Base class for Enterprise Customer views.
@@ -114,7 +148,7 @@ class BaseEnterpriseCustomerView(View):
         """
         opts = customer._meta
         codename = get_permission_codename('change', opts)
-        has_change_permission = request.user.has_perm('%s.%s' % (opts.app_label, codename))
+        has_change_permission = request.user.has_perm('{}.{}'.format(opts.app_label, codename))
         return {
             'has_change_permission': has_change_permission,
             'opts': opts
@@ -125,7 +159,7 @@ class BaseEnterpriseCustomerView(View):
         Return the default context parameters
         """
         return {
-            'enterprise_customer': EnterpriseCustomer.objects.get(uuid=customer_uuid)  # pylint: disable=no-member
+            'enterprise_customer': EnterpriseCustomer.objects.get(uuid=customer_uuid)
         }
 
     def _build_context(self, request, customer_uuid):
@@ -142,7 +176,8 @@ class BaseEnterpriseCustomerView(View):
         render the form with appropriate context.
         """
         context = self._build_context(request, customer_uuid)
-        context.update(additional_context)
+        if additional_context:
+            context.update(additional_context)
         return render(request, self.template, context)
 
 
@@ -300,7 +335,7 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
         ENROLLMENT_URL = 'ENROLLMENT_API_ROOT_URL'
 
     def _get_view_context(self, request, customer_uuid):
-        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)  # pylint: disable=no-member
+        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)
         search_keyword = self.get_search_keyword(request)
         linked_learners = self.get_enterprise_customer_user_queryset(request, search_keyword, customer_uuid)
         pending_linked_learners = self.get_pending_users_queryset(search_keyword, customer_uuid)
@@ -322,7 +357,11 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
 
     def get_enterprise_customer_user_queryset(self, request, search_keyword, customer_uuid, page_size=PAGE_SIZE):
         """
-        Get the list of EnterpriseCustomerUsers we want to render.
+        Get the list of EnterpriseCustomerUsers we want to render. Note that,
+        if no ``search_keyword`` is provided, the resulting queryset will
+        only include up to some constant limit of results, so that we
+        can support searching for particular user records in customers
+        with very large numbers of learners.
 
         Arguments:
             request (HttpRequest): HTTP Request instance.
@@ -332,16 +371,29 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
             page_size (int): Number of learners displayed in each paginated set.
         """
         page = request.GET.get('page', 1)
-        learners = EnterpriseCustomerUser.objects.filter(enterprise_customer__uuid=customer_uuid)
-        user_ids = learners.values_list('user_id', flat=True)
-        matching_users = User.objects.filter(pk__in=user_ids)
-        if search_keyword is not None:
-            matching_users = matching_users.filter(
-                Q(email__icontains=search_keyword) | Q(username__icontains=search_keyword)
+        if search_keyword:
+            enterprise_learners = self._get_searched_enterprise_customer_users(
+                search_keyword,
+                customer_uuid,
             )
-        matching_user_ids = matching_users.values_list('pk', flat=True)
-        learners = learners.filter(user_id__in=matching_user_ids)
-        return paginated_list(learners, page, page_size)
+        else:
+            enterprise_learners = EnterpriseCustomerUser.objects.filter(
+                enterprise_customer__uuid=customer_uuid,
+            )[:DJANGO_ADMIN_MANAGE_LEARNERS_LIMIT]
+        return paginated_list(enterprise_learners, page, page_size)
+
+    def _get_searched_enterprise_customer_users(self, search_keyword, customer_uuid):
+        """
+        Helper to return a (possibly truncated) queryset of enterprise customer users
+        who match some search criteria based on corresponding ``core.User`` records.
+        """
+        matching_user_ids = User.objects.filter(
+            Q(email__icontains=search_keyword) | Q(username__icontains=search_keyword)
+        ).values_list('id', flat=True)
+        return EnterpriseCustomerUser.objects.filter(
+            enterprise_customer__uuid=customer_uuid,
+            user_id__in=matching_user_ids,
+        )[:DJANGO_ADMIN_MANAGE_LEARNERS_LIMIT]
 
     def get_pending_users_queryset(self, search_keyword, customer_uuid):
         """
@@ -352,14 +404,15 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
             customer_uuid (str): A unique identifier to filter down to only pending users
             linked to a particular EnterpriseCustomer.
         """
-        queryset = PendingEnterpriseCustomerUser.objects.filter(
-            enterprise_customer__uuid=customer_uuid
-        )
-
-        if search_keyword is not None:
-            queryset = queryset.filter(user_email__icontains=search_keyword)
-
-        return queryset
+        if search_keyword:
+            return PendingEnterpriseCustomerUser.objects.filter(
+                enterprise_customer__uuid=customer_uuid,
+                user_email__icontains=search_keyword,
+            )[:DJANGO_ADMIN_MANAGE_LEARNERS_LIMIT]
+        else:
+            return PendingEnterpriseCustomerUser.objects.filter(
+                enterprise_customer__uuid=customer_uuid
+            )[:DJANGO_ADMIN_MANAGE_LEARNERS_LIMIT]
 
     @classmethod
     def _handle_singular(cls, request, enterprise_customer, manage_learners_form):
@@ -380,11 +433,11 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
                 ValidationMessages.INVALID_EMAIL_OR_USERNAME,
                 raise_exception=False,
             )
-        except ValidationError as exc:
-            manage_learners_form.add_error(ManageLearnersForm.Fields.EMAIL_OR_USERNAME, exc)
-        else:
-            EnterpriseCustomerUser.objects.link_user(enterprise_customer, email)
+            EnterpriseCustomerUser.all_objects.link_user(enterprise_customer, email)
             return [email]
+        except (ValidationError, LinkUserToEnterpriseError) as exc:
+            manage_learners_form.add_error(ManageLearnersForm.Fields.EMAIL_OR_USERNAME, exc)
+            return None
 
     def _handle_bulk_upload_errors(self, manage_learners_form, errors):
         """
@@ -421,7 +474,7 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
         Returns: List of processable email addresses.
         """
         count = len(emails)
-        messages.success(request, ungettext(
+        messages.success(request, ngettext(
             "{count} new learner was added to {enterprise_customer_name}.",
             "{count} new learners were added to {enterprise_customer_name}.",
             count
@@ -520,15 +573,20 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
         except ValidationError as exc:
             errors.append(exc)
 
+        # do the actual linking if there are no errors at this point:
+        if not errors:
+            try:
+                with transaction.atomic():
+                    for email in emails:
+                        EnterpriseCustomerUser.all_objects.link_user(enterprise_customer, email)
+            except LinkUserToEnterpriseError as exc:
+                errors.append(exc)
+
         if errors:
             cls._handle_bulk_upload_errors(cls, manage_learners_form=manage_learners_form, errors=errors)
             # There were validation errors, so prevent any further action.
             return [], {}
 
-        # There were no errors. Now do the actual linking:
-        for email in emails:
-            EnterpriseCustomerUser.objects.link_user(enterprise_customer, email)
-
         # Process the bulk uploaded data:
         processable_emails = cls._process_bulk_upload_data(
             cls,
@@ -570,7 +628,7 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
         enrolled_count = len(users)
         return (
             'success',
-            ungettext(
+            ngettext(
                 '{enrolled_count} learner was enrolled in {enrolled_in}.',
                 '{enrolled_count} learners were enrolled in {enrolled_in}.',
                 enrolled_count,
@@ -709,7 +767,7 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
         Returns:
             django.http.response.HttpResponse: HttpResponse
         """
-        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)  # pylint: disable=no-member
+        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)
         manage_learners_form = ManageLearnersForm(user=request.user, enterprise_customer=enterprise_customer)
 
         return self.get_form_view(
@@ -731,7 +789,7 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
         Returns:
             django.http.response.HttpResponse: HttpResponse
         """
-        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)  # pylint: disable=no-member
+        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)
         manage_learners_form = ManageLearnersForm(
             request.POST,
             request.FILES,
@@ -772,7 +830,7 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
                 manual_enrollment_reason = manage_learners_form.cleaned_data.get(ManageLearnersForm.Fields.REASON)
             else:
                 manual_enrollment_reason = None
-                logging.exception(
+                LOG.exception(
                     "To create enrollment audits for enterprise learners, "
                     "this package must be installed in an Open edX environment."
                 )
@@ -800,8 +858,8 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
                         discount=discount
                     )
             else:
-                for course_id in course_id_with_emails:
-                    emails_to_enroll = course_id_with_emails[course_id]
+                for course_id, emails in course_id_with_emails.items():
+                    emails_to_enroll = emails
                     if emails_to_enroll:
                         self._enroll_users(
                             request=request,
@@ -843,11 +901,12 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
             django.http.response.HttpResponse: HttpResponse
         """
         # TODO: pylint acts stupid - find a way around it without suppressing
-        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)  # pylint: disable=no-member
+        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)
         email_to_unlink = request.GET["unlink_email"]
         try:
             EnterpriseCustomerUser.objects.unlink_user(
-                enterprise_customer=enterprise_customer, user_email=email_to_unlink
+                enterprise_customer=enterprise_customer,
+                user_email=email_to_unlink,
             )
         except (EnterpriseCustomerUser.DoesNotExist, PendingEnterpriseCustomerUser.DoesNotExist):
             message = _("Email {email} is not associated with Enterprise "
@@ -857,3 +916,71 @@ class EnterpriseCustomerManageLearnersView(BaseEnterpriseCustomerView):
             return HttpResponse(message, content_type="application/json", status=404)
 
         return JsonResponse({})
+
+
+class EnterpriseCustomerSetupAuthOrgIDView(BaseEnterpriseCustomerView):
+    """
+    Setup Auth org id View.
+
+    This action will configure SSO to GetSmarter using edX credentials via Auth0.
+    """
+    template = 'enterprise/admin/setup_auth_org_id.html'
+
+    def get(self, request, customer_uuid):
+        """
+        Handle GET request - render "Setup Auth org id" form.
+
+        Arguments:
+            request (django.http.request.HttpRequest): Request instance
+            customer_uuid (str): Enterprise Customer UUID
+
+        Returns:
+            django.http.response.HttpResponse: HttpResponse
+        """
+        if not get_sso_orchestrator_configure_edx_oauth_path():
+            return HttpResponseForbidden(
+                "The ENTERPRISE_SSO_ORCHESTRATOR_CONFIGURE_EDX_OAUTH_PATH setting was not configured."
+            )
+        return self.get_form_view(request, customer_uuid)
+
+    def post(self, request, customer_uuid):
+        """
+        Handle POST request - handle form submissions.
+
+        Arguments:
+            request (django.http.request.HttpRequest): Request instance
+            customer_uuid (str): Enterprise Customer UUID
+        """
+        if not get_sso_orchestrator_configure_edx_oauth_path():
+            return HttpResponseForbidden(
+                "The ENTERPRISE_SSO_ORCHESTRATOR_CONFIGURE_EDX_OAUTH_PATH setting was not configured."
+            )
+
+        enterprise_customer = EnterpriseCustomer.objects.get(uuid=customer_uuid)
+
+        # Call the configure-edx-oauth endpoint on the enterprise-sso-orchestrator service to obtain an orgId.
+        # This will raise SsoOrchestratorClientError if the API request fails.
+        try:
+            auth_org_id = EnterpriseSSOOrchestratorApiClient().configure_edx_oauth(enterprise_customer)
+        except SsoOrchestratorClientError as exc:
+            error_msg = (
+                f"Error configuring edx oauth for enterprise customer {enterprise_customer.name}"
+                f"<{enterprise_customer.uuid}>: {exc}"
+            )
+            LOG.exception(error_msg)
+            return HttpResponseServerError(error_msg)
+
+        if auth_org_id:
+            enterprise_customer.auth_org_id = auth_org_id
+            enterprise_customer.save()
+            messages.success(request, _('Successfully written the "Auth org id" field for this enterprise customer.'))
+            return HttpResponseRedirect(reverse("admin:" + UrlNames.SETUP_AUTH_ORG_ID, args=(customer_uuid,)))
+        else:
+            # Annoyingly, there's still the remote possibility that the request succeeded but we failed to retrieve the
+            # auth_org_id. This might be due to a regression in the API response schema.
+            error_msg = (
+                f"Error configuring edx oauth for enterprise customer {enterprise_customer.name}"
+                f"<{enterprise_customer.uuid}>: Missing orgId."
+            )
+            LOG.exception(error_msg)
+            return HttpResponseServerError(error_msg)
