@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """ Custom API permissions. """
 
 from rest_framework import permissions
@@ -10,8 +8,8 @@ class IsInEnterpriseGroup(permissions.BasePermission):
     Find out if the requesting user belongs to a django group meant for granting access to an enterprise feature.
     This check applies to both staff and non-staff users.
     """
-    ALLOWED_API_GROUPS = []  # pylint: disable=invalid-name
-    message = u'User is not allowed to access the view.'
+    ALLOWED_API_GROUPS = []
+    message = 'User is not allowed to access the view.'
 
     def has_permission(self, request, view):
         return (
