@@ -1,48 +1,54 @@
-# -*- coding: utf-8 -*-
 """
 Client for communicating with the E-Commerce API.
 """
 
 import logging
+from urllib.parse import urljoin
 
-from edx_rest_api_client.client import EdxRestApiClient
-from requests.exceptions import ConnectionError, Timeout  # pylint: disable=redefined-builtin
-from slumber.exceptions import SlumberBaseException
+from requests.exceptions import ConnectionError, RequestException, Timeout  # pylint: disable=redefined-builtin
 
 from django.conf import settings
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
+from enterprise.api_client.client import NoAuthAPIClient
 from enterprise.utils import NotConnectedToOpenEdX, format_price
 
 try:
-    from openedx.core.djangoapps.commerce.utils import ecommerce_api_client
+    from openedx.core.djangoapps.site_configuration import helpers as configuration_helpers
 except ImportError:
-    ecommerce_api_client = None
+    configuration_helpers = None
+
+try:
+    from openedx.core.djangoapps.commerce.utils import get_ecommerce_api_client
+except ImportError:
+    get_ecommerce_api_client = None
 
 LOGGER = logging.getLogger(__name__)
 
 
 class EcommerceApiClient:
     """
-    Object builds an API client to make calls to the E-Commerce API.
+    The API client to make calls to the E-Commerce API.
     """
-
     def __init__(self, user):
         """
-        Create an E-Commerce API client, authenticated with the API token from Django settings.
+        Create an E-Commerce API client, authenticated with the API token.
 
         This method retrieves an authenticated API client that can be used
         to access the ecommerce API. It raises an exception to be caught at
         a higher level if the package doesn't have OpenEdX resources available.
         """
-        if ecommerce_api_client is None:
+        if get_ecommerce_api_client is None or configuration_helpers is None:
             raise NotConnectedToOpenEdX(
                 _('To get a ecommerce_api_client, this package must be '
                   'installed in an Open edX environment.')
             )
 
         self.user = user
-        self.client = ecommerce_api_client(user)
+        self.client = get_ecommerce_api_client(user)
+        self.API_BASE_URL = configuration_helpers.get_value(  # pylint: disable=invalid-name
+            'ECOMMERCE_API_URL', settings.ECOMMERCE_API_URL
+        )
 
     def get_course_final_price(self, mode, currency='$', enterprise_catalog_uuid=None):
         """
@@ -50,15 +56,20 @@ class EcommerceApiClient:
 
         Returns:
             str: Discounted price of the course mode.
-
         """
+        api_url = urljoin(f"{self.API_BASE_URL}/", "baskets/calculate/")
         try:
-            price_details = self.client.baskets.calculate.get(
-                sku=[mode['sku']],
-                username=self.user.username,
-                catalog=enterprise_catalog_uuid,
+            response = self.client.get(
+                api_url,
+                params={
+                    "sku": [mode['sku']],
+                    "username": self.user.username,
+                    "catalog": enterprise_catalog_uuid,
+                }
             )
-        except (SlumberBaseException, ConnectionError, Timeout) as exc:
+            response.raise_for_status()
+            price_details = response.json()
+        except (RequestException, ConnectionError, Timeout) as exc:
             LOGGER.exception('Failed to get price details for sku %s due to: %s', mode['sku'], str(exc))
             price_details = {}
         price = price_details.get('total_incl_tax', mode['min_price'])
@@ -70,22 +81,23 @@ class EcommerceApiClient:
         """
         Calls ecommerce to create orders for the manual enrollments passed in.
 
-        `enrollments` should be a list of enrollments with the following format:
-        {
-            "lms_user_id": <int>,
-            "username": <str>,
-            "email": <str>,
-            "course_run_key": <str>
-        }
+        `enrollments` should be a list of enrollments with the following format::
 
-        Since `student.CourseEnrollment` lives in LMS, we're just passing around dicts of the relevant information.
+            {
+                "lms_user_id": <int>,
+                "username": <str>,
+                "email": <str>,
+                "course_run_key": <str>
+            }
+
+        Since `student.CourseEnrollment` lives in LMS, we're just passing
+        around dicts of the relevant information.
         """
+        api_url = urljoin(f"{self.API_BASE_URL}/", "manual_course_enrollment_order/")
         try:
-            order_response = self.client.manual_course_enrollment_order.post(
-                {
-                    "enrollments": enrollments
-                }
-            )
+            response = self.client.post(api_url, json={"enrollments": enrollments})
+            response.raise_for_status()
+            order_response = response.json()
             order_creations = order_response["orders"]
             successful_creations = [order for order in order_creations if order["status"] == "success"]
             failed_creations = [order for order in order_creations if order["status"] == "failure"]
@@ -99,7 +111,7 @@ class EcommerceApiClient:
                     "Failed to created orders for the following manual enrollments. %s",
                     failed_creations
                 )
-        except (SlumberBaseException, ConnectionError, Timeout) as exc:
+        except (RequestException, ConnectionError, Timeout) as exc:
             LOGGER.exception(
                 "Failed to create order for manual enrollments for the following enrollments: %s. Reason: %s",
                 enrollments,
@@ -107,25 +119,9 @@ class EcommerceApiClient:
             )
 
 
-class NoAuthEcommerceClient:
+class NoAuthEcommerceClient(NoAuthAPIClient):
     """
-    Class to build an E-Commerce client to make calls to the E-Commerce.
+    The E-Commerce API client without authentication.
     """
-
     API_BASE_URL = settings.ECOMMERCE_PUBLIC_URL_ROOT
     APPEND_SLASH = False
-
-    def __init__(self):
-        """
-        Create an E-Commerce client.
-        """
-        self.client = EdxRestApiClient(self.API_BASE_URL, append_slash=self.APPEND_SLASH)
-
-    def get_health(self):
-        """
-        Retrieve health details for E-Commerce service.
-
-        Returns:
-            dict: Response containing E-Commerce service health.
-        """
-        return self.client.health.get()
