@@ -1,108 +1,31 @@
-# -*- coding: utf-8 -*-
 """
 Utilities to get details from the course catalog API.
 """
 
-import datetime
 import logging
-from functools import wraps
-from time import time
+import time
+from urllib.parse import urljoin
 
-from edx_rest_api_client.client import EdxRestApiClient
 from opaque_keys.edx.keys import CourseKey
-from requests.exceptions import ConnectionError, Timeout  # pylint: disable=redefined-builtin
-from slumber.exceptions import HttpNotFoundError, SlumberBaseException
+from requests.exceptions import (  # pylint: disable=redefined-builtin
+    ConnectionError,
+    HTTPError,
+    RequestException,
+    Timeout,
+)
 
 from django.conf import settings
-from django.utils import timezone
 
+from enterprise.api_client.client import BackendServiceAPIClient, NoAuthAPIClient, UserAPIClient
 from enterprise.constants import COURSE_MODE_SORT_ORDER, EXCLUDED_COURSE_MODES
-from enterprise.utils import NotConnectedToOpenEdX, get_enterprise_worker_user
 
 try:
     from openedx.core.djangoapps.embargo import api as embargo_api
 except ImportError:
     embargo_api = None
 
-try:
-    from openedx.core.djangoapps.oauth_dispatch import jwt as JwtBuilder
-except ImportError:
-    JwtBuilder = None
-
 
 LOGGER = logging.getLogger(__name__)
-LMS_API_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
-LMS_API_DATETIME_FORMAT_WITHOUT_TIMEZONE = '%Y-%m-%dT%H:%M:%S'
-
-
-class NoAuthenticationLmsApiClient:
-    """
-    Object builds an API client to make calls to the edxapp LMS API.
-
-    Authentication is not required.
-    """
-
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/'
-    APPEND_SLASH = False
-
-    def __init__(self):
-        """
-        Create an LMS API client.
-        """
-        self.client = EdxRestApiClient(self.API_BASE_URL, append_slash=self.APPEND_SLASH)
-
-
-class JwtLmsApiClient:
-    """
-    LMS client authenticates using a JSON Web Token (JWT) for the given user.
-    """
-
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/'
-    APPEND_SLASH = False
-
-    def __init__(self, user, expires_in=settings.OAUTH_ID_TOKEN_EXPIRATION):
-        """
-        Connect to the REST API.
-        """
-        self.user = user
-        self.expires_in = expires_in
-        self.expires_at = 0
-        self.client = None
-
-    def connect(self):
-        """
-        Connect to the REST API, authenticating with a JWT for the current user.
-        """
-        if JwtBuilder is None:
-            raise NotConnectedToOpenEdX("This package must be installed in an OpenEdX environment.")
-
-        now = int(time())
-        jwt = JwtBuilder.create_jwt_for_user(self.user)
-        self.client = EdxRestApiClient(
-            self.API_BASE_URL, append_slash=self.APPEND_SLASH, jwt=jwt,
-        )
-        self.expires_at = now + self.expires_in
-
-    def token_expired(self):
-        """
-        Return True if the JWT token has expired, False if not.
-        """
-        return int(time()) > self.expires_at
-
-    @staticmethod
-    def refresh_token(func):
-        """
-        Use this method decorator to ensure the JWT token is refreshed when needed.
-        """
-        @wraps(func)
-        def inner(self, *args, **kwargs):
-            """
-            Before calling the wrapped function, we check if the JWT token is expired, and if so, re-connect.
-            """
-            if self.token_expired():
-                self.connect()
-            return func(self, *args, **kwargs)
-        return inner
 
 
 class EmbargoApiClient:
@@ -111,34 +34,28 @@ class EmbargoApiClient:
     """
 
     @staticmethod
-    def redirect_if_blocked(course_run_ids, user=None, ip_address=None, url=None):
+    def redirect_if_blocked(request, course_run_ids, user=None):
         """
         Return redirect to embargo error page if the given user is blocked.
         """
         for course_run_id in course_run_ids:
             redirect_url = embargo_api.redirect_if_blocked(
-                CourseKey.from_string(course_run_id),
+                request=request,
+                course_key=CourseKey.from_string(course_run_id),
                 user=user,
-                ip_address=ip_address,
-                url=url
             )
             if redirect_url:
                 return redirect_url
         return None
 
 
-class EnrollmentApiClient(JwtLmsApiClient):
+class EnrollmentApiClient(BackendServiceAPIClient):
     """
-    Object builds an API client to make calls to the Enrollment API.
+    The API client to make calls to the Enrollment API.
     """
 
     API_BASE_URL = settings.ENTERPRISE_ENROLLMENT_API_URL
 
-    def __init__(self, user=None):
-        user = user if user else get_enterprise_worker_user()
-        super().__init__(user)
-
-    @JwtLmsApiClient.refresh_token
     def get_course_details(self, course_id):
         """
         Query the Enrollment API for the course details of the given course_id.
@@ -149,12 +66,14 @@ class EnrollmentApiClient(JwtLmsApiClient):
         Returns:
             dict: A dictionary containing details about the course, in an enrollment context (allowed modes, etc.)
         """
+        api_url = self.get_api_url(f"course/{course_id}")
         try:
-            return self.client.course(course_id).get()
-        except (SlumberBaseException, ConnectionError, Timeout) as exc:
+            response = self.client.get(api_url)
+            response.raise_for_status()
+            return response.json()
+        except (RequestException, ConnectionError, Timeout):
             LOGGER.exception(
-                'Failed to retrieve course enrollment details for course [%s] due to: [%s]',
-                course_id, str(exc)
+                'Failed to retrieve course enrollment details for course [%s].', course_id
             )
             return {}
 
@@ -180,7 +99,6 @@ class EnrollmentApiClient(JwtLmsApiClient):
         # Sort slug weights in descending order
         return sorted(modes, key=slug_weight, reverse=True)
 
-    @JwtLmsApiClient.refresh_token
     def get_course_modes(self, course_id):
         """
         Query the Enrollment API for the specific course modes that are available for the given course_id.
@@ -196,7 +114,6 @@ class EnrollmentApiClient(JwtLmsApiClient):
         modes = details.get('course_modes', [])
         return self._sort_course_modes([mode for mode in modes if mode['slug'] not in EXCLUDED_COURSE_MODES])
 
-    @JwtLmsApiClient.refresh_token
     def has_course_mode(self, course_run_id, mode):
         """
         Query the Enrollment API to see whether a course run has a given course mode available.
@@ -211,8 +128,7 @@ class EnrollmentApiClient(JwtLmsApiClient):
         course_modes = self.get_course_modes(course_run_id)
         return any(course_mode for course_mode in course_modes if course_mode['slug'] == mode)
 
-    @JwtLmsApiClient.refresh_token
-    def enroll_user_in_course(self, username, course_id, mode, cohort=None):
+    def enroll_user_in_course(self, username, course_id, mode, cohort=None, enterprise_uuid=None):
         """
         Call the enrollment API to enroll the user in the course specified by course_id.
 
@@ -221,22 +137,27 @@ class EnrollmentApiClient(JwtLmsApiClient):
             course_id (str): The string value of the course's unique identifier
             mode (str): The enrollment mode which should be used for the enrollment
             cohort (str): Add the user to this named cohort
+            enterprise_uuid (str): Add course enterprise uuid
 
         Returns:
             dict: A dictionary containing details of the enrollment, including course details, mode, username, etc.
 
         """
-        return self.client.enrollment.post(
-            {
+        api_url = self.get_api_url("enrollment")
+        response = self.client.post(
+            api_url,
+            json={
                 'user': username,
                 'course_details': {'course_id': course_id},
                 'is_active': True,
                 'mode': mode,
                 'cohort': cohort,
+                'enterprise_uuid': str(enterprise_uuid)
             }
         )
+        response.raise_for_status()
+        return response.json()
 
-    @JwtLmsApiClient.refresh_token
     def unenroll_user_from_course(self, username, course_id):
         """
         Call the enrollment API to unenroll the user in the course specified by course_id.
@@ -248,17 +169,21 @@ class EnrollmentApiClient(JwtLmsApiClient):
         """
         enrollment = self.get_course_enrollment(username, course_id)
         if enrollment and enrollment['is_active']:
-            response = self.client.enrollment.post({
-                'user': username,
-                'course_details': {'course_id': course_id},
-                'is_active': False,
-                'mode': enrollment['mode']
-            })
-            return not response['is_active']
+            api_url = self.get_api_url("enrollment")
+            response = self.client.post(
+                api_url,
+                json={
+                    'user': username,
+                    'course_details': {'course_id': course_id},
+                    'is_active': False,
+                    'mode': enrollment['mode']
+                }
+            )
+            response.raise_for_status()
+            return not response.json()['is_active']
 
         return False
 
-    @JwtLmsApiClient.refresh_token
     def update_course_enrollment_mode_for_user(self, username, course_id, mode):
         """
         Call the enrollment API to update a user's course enrollment to the specified mode, e.g. "audit".
@@ -271,13 +196,18 @@ class EnrollmentApiClient(JwtLmsApiClient):
         Returns:
             dict: A dictionary containing details of the enrollment, including course details, mode, username, etc.
         """
-        return self.client.enrollment.post({
-            'user': username,
-            'course_details': {'course_id': course_id},
-            'mode': mode,
-        })
+        api_url = self.get_api_url("enrollment")
+        response = self.client.post(
+            api_url,
+            json={
+                'user': username,
+                'course_details': {'course_id': course_id},
+                'mode': mode,
+            }
+        )
+        response.raise_for_status()
+        return response.json()
 
-    @JwtLmsApiClient.refresh_token
     def get_course_enrollment(self, username, course_id):
         """
         Query the enrollment API to get information about a single course enrollment.
@@ -290,29 +220,29 @@ class EnrollmentApiClient(JwtLmsApiClient):
             dict: A dictionary containing details of the enrollment, including course details, mode, username, etc.
 
         """
-        endpoint = getattr(
-            self.client.enrollment,
-            '{username},{course_id}'.format(username=username, course_id=course_id)
-        )
+        username_course_string = '{username},{course_id}'.format(username=username, course_id=course_id)
+        api_url = self.get_api_url(f"enrollment/{username_course_string}")
+
         try:
-            result = endpoint.get()
-        except HttpNotFoundError:
+            response = self.client.get(api_url)
+            response.raise_for_status()
+        except HTTPError as err:
             # This enrollment data endpoint returns a 404 if either the username or course_id specified isn't valid
-            LOGGER.error(
-                'Course enrollment details not found for invalid username or course; username=[%s], course=[%s]',
-                username,
-                course_id
-            )
+            if err.response.status_code == 404:
+                LOGGER.error(
+                    'Course enrollment details not found for invalid username or course; username=[%s], course=[%s]',
+                    username,
+                    course_id
+                )
             return None
         # This enrollment data endpoint returns an empty string if the username and course_id is valid, but there's
         # no matching enrollment found
-        if not result:
+        if not response.content:
             LOGGER.info('Failed to find course enrollment details for user [%s] and course [%s]', username, course_id)
             return None
 
-        return result
+        return response.json()
 
-    @JwtLmsApiClient.refresh_token
     def is_enrolled(self, username, course_run_id):
         """
         Query the enrollment API and determine if a learner is enrolled in a course run.
@@ -328,7 +258,6 @@ class EnrollmentApiClient(JwtLmsApiClient):
         enrollment = self.get_course_enrollment(username, course_run_id)
         return enrollment is not None and enrollment.get('is_active', False)
 
-    @JwtLmsApiClient.refresh_token
     def get_enrolled_courses(self, username):
         """
         Query the enrollment API to get a list of the courses a user is enrolled in.
@@ -340,15 +269,18 @@ class EnrollmentApiClient(JwtLmsApiClient):
             list: A list of course objects, along with relevant user-specific enrollment details.
 
         """
-        return self.client.enrollment.get(user=username)
+        api_url = self.get_api_url("enrollment")
+        response = self.client.get(api_url, params={"user": username})
+        response.raise_for_status()
+        return response.json()
 
 
-class CourseApiClient(NoAuthenticationLmsApiClient):
+class CourseApiClient(NoAuthAPIClient):
     """
-    Object builds an API client to make calls to the Course API.
+    The API client to make calls to the Course API.
     """
 
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/courses/v1/'
+    API_BASE_URL = urljoin(f"{settings.LMS_INTERNAL_ROOT_URL}/", "api/courses/v1/")
     APPEND_SLASH = True
 
     def get_course_details(self, course_id):
@@ -361,21 +293,24 @@ class CourseApiClient(NoAuthenticationLmsApiClient):
         Returns:
             dict: Contains keys identifying those course details available from the courses API (e.g., name).
         """
+        api_url = self.get_api_url(f"courses/{course_id}")
         try:
-            return self.client.courses(course_id).get()
-        except (SlumberBaseException, ConnectionError, Timeout) as exc:
-            LOGGER.exception('Details not found for course [%s] due to: [%s]', course_id, str(exc))
+            response = self.client.get(api_url)
+            response.raise_for_status()
+            return response.json()
+        except (RequestException, ConnectionError, Timeout):
+            LOGGER.exception('Details not found for course [%s].', course_id)
             return None
 
 
-class ThirdPartyAuthApiClient(JwtLmsApiClient):
+class ThirdPartyAuthApiClient(UserAPIClient):
     """
-    Object builds an API client to make calls to the Third Party Auth API.
+    The API client to make calls to the Third Party Auth API.
     """
 
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/third_party_auth/v0/'
+    API_BASE_URL = urljoin(f"{settings.LMS_INTERNAL_ROOT_URL}/", "api/third_party_auth/v0/")
 
-    @JwtLmsApiClient.refresh_token
+    @UserAPIClient.refresh_token
     def get_remote_id(self, identity_provider, username):
         """
         Retrieve the remote identifier for the given username.
@@ -389,7 +324,7 @@ class ThirdPartyAuthApiClient(JwtLmsApiClient):
         """
         return self._get_results(identity_provider, 'username', username, 'remote_id')
 
-    @JwtLmsApiClient.refresh_token
+    @UserAPIClient.refresh_token
     def get_username_from_remote_id(self, identity_provider, remote_id):
         """
         Retrieve the remote identifier for the given username.
@@ -407,19 +342,23 @@ class ThirdPartyAuthApiClient(JwtLmsApiClient):
         """
         Calls the third party auth api endpoint to get the mapping between usernames and remote ids.
         """
+        api_url = self.get_api_url(f"providers/{identity_provider}/users")
         try:
             kwargs = {param_name: param_value}
-            returned = self.client.providers(identity_provider).users.get(**kwargs)
-            results = returned.get('results', [])
-        except HttpNotFoundError:
-            LOGGER.error(
-                'username not found for third party provider={provider}, {querystring_param}={id}'.format(
-                    provider=identity_provider,
-                    querystring_param=param_name,
-                    id=param_value
+            response = self.client.get(api_url, params=kwargs)
+            response.raise_for_status()
+            results = response.json().get('results', [])
+        except HTTPError as err:
+            if err.response.status_code == 404:
+                LOGGER.error(
+                    'Username not found for third party provider={%s}, {%s}={%s}',
+                    identity_provider,
+                    param_name,
+                    param_value
                 )
-            )
-            results = []
+                results = []
+            else:
+                raise
 
         for row in results:
             if row.get(param_name) == param_value:
@@ -427,17 +366,24 @@ class ThirdPartyAuthApiClient(JwtLmsApiClient):
         return None
 
 
-class GradesApiClient(JwtLmsApiClient):
+class GradesApiClient(UserAPIClient):
     """
-    Object builds an API client to make calls to the LMS Grades API.
+    The API client to make calls to the LMS Grades API.
 
     Note that this API client requires a JWT token, and so it keeps its token alive.
     """
 
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/grades/v1/'
+    MAX_RETRIES = getattr(settings, "ENTERPRISE_DEGREED2_MAX_RETRIES", 4)
+    API_BASE_URL = urljoin(f"{settings.LMS_INTERNAL_ROOT_URL}/", "api/grades/v1/")
     APPEND_SLASH = True
 
-    @JwtLmsApiClient.refresh_token
+    def _calculate_backoff(self, attempt_count):
+        """
+        Calculate the seconds to sleep based on attempt_count
+        """
+        return (self.BACKOFF_FACTOR * (2 ** (attempt_count - 1)))
+
+    @UserAPIClient.refresh_token
     def get_course_grade(self, course_id, username):
         """
         Retrieve the grade for the given username for the given course_id.
@@ -448,7 +394,7 @@ class GradesApiClient(JwtLmsApiClient):
 
         Raises:
 
-        HttpNotFoundError if no grade found for the given user+course.
+        HTTPError if no grade found for the given user+course.
 
         Returns:
 
@@ -461,14 +407,16 @@ class GradesApiClient(JwtLmsApiClient):
         * ``letter_grade``: A letter grade as defined in grading_policy (e.g. 'A' 'B' 'C' for 6.002x) or None
 
         """
-        results = self.client.courses(course_id).get(username=username)
-        for row in results:
+        api_url = self.get_api_url(f"courses/{course_id}")
+        response = self.client.get(api_url, params={"username": username})
+        response.raise_for_status()
+        for row in response.json():
             if row.get('username') == username:
                 return row
 
-        raise HttpNotFoundError('No grade record found for course={}, username={}'.format(course_id, username))
+        raise HTTPError(f'No grade record found for course={course_id}, username={username}')
 
-    @JwtLmsApiClient.refresh_token
+    @UserAPIClient.refresh_token
     def get_course_assessment_grades(self, course_id, username):
         """
         Retrieve the assessment grades for the given username for the given course_id.
@@ -479,7 +427,7 @@ class GradesApiClient(JwtLmsApiClient):
 
         Raises:
 
-        HttpNotFoundError if no grade found for the given user+course.
+        HTTPError if no grade found for the given user+course.
 
         Returns:
 
@@ -494,28 +442,46 @@ class GradesApiClient(JwtLmsApiClient):
         * ``percent``: A float representing the overall grade for the course.
         * ``module_id``: The ID of the subsection.
         """
-        results = self.client.gradebook(course_id).get(user_contains=username).get('results')
-        for row in results:
-            if row.get('username') == username:
-                return row.get('section_breakdown')
+        attempts = 0
+        while True:
+            attempts = attempts + 1
+            api_url = self.get_api_url(f"gradebook/{course_id}")
+            try:
+                response = self.client.get(api_url, params={"username": username}, timeout=40)
+                response.raise_for_status()
+                break
+            except Timeout as to_exception:
+                if attempts <= self.MAX_RETRIES:
+                    sleep_seconds = self._calculate_backoff(attempts)
+                    LOGGER.warning(
+                        f"[ATTEMPT: {attempts}] Request to the LMS grades API timeouted out with "
+                        f"exception: {to_exception}, backing off for {sleep_seconds} seconds and retrying"
+                    )
+                    time.sleep(sleep_seconds)
+                else:
+                    LOGGER.warning(
+                        f"Requests to the grades API has reached the max number of retries [{self.MAX_RETRIES}], "
+                        f"attempting to retrieve grade data for learner: {username} under course {course_id}"
+                    )
+                    raise to_exception
 
-        raise HttpNotFoundError('No assessment grade record found for course={}, username={}'.format(
-            course_id,
-            username
-        ))
+        results = response.json()
+        if results.get('username') == username:
+            return results.get('section_breakdown')
+
+        raise HTTPError(f"No assessment grade record found for course={course_id}, username={username}")
 
 
-class CertificatesApiClient(JwtLmsApiClient):
+class CertificatesApiClient(UserAPIClient):
     """
-    Object builds an API client to make calls to the LMS Certificates API.
+    The API client to make calls to the LMS Certificates API.
 
     Note that this API client requires a JWT token, and so it keeps its token alive.
     """
-
-    API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL + '/api/certificates/v0/'
+    API_BASE_URL = urljoin(f"{settings.LMS_INTERNAL_ROOT_URL}/", "api/certificates/v0/")
     APPEND_SLASH = True
 
-    @JwtLmsApiClient.refresh_token
+    @UserAPIClient.refresh_token
     def get_course_certificate(self, course_id, username):
         """
         Retrieve the certificate for the given username for the given course_id.
@@ -524,10 +490,6 @@ class CertificatesApiClient(JwtLmsApiClient):
         * ``course_id`` (str): The string value of the course's unique identifier
         * ``username`` (str): The username ID identifying the user for which to retrieve the certificate
 
-        Raises:
-
-        HttpNotFoundError if no certificate found for the given user+course.
-
         Returns:
 
         a dict containing:
@@ -535,19 +497,22 @@ class CertificatesApiClient(JwtLmsApiClient):
         * ``username``: A string representation of an user's username passed in the request.
         * ``course_id``: A string representation of a Course ID.
         * ``certificate_type``: A string representation of the certificate type.
-        * ``created_date`: Datetime the certificate was created (tz-aware).
+        * ``created_date``: Datetime the certificate was created (tz-aware).
         * ``status``: A string representation of the certificate status.
         * ``is_passing``: True if the certificate has a passing status, False if not.
         * ``download_url``: A string representation of the certificate url.
         * ``grade``: A string representation of a float for the user's course grade.
 
         """
-        return self.client.certificates(username).courses(course_id).get()
+        api_url = self.get_api_url(f"certificates/{username}/courses/{course_id}")
+        response = self.client.get(api_url)
+        response.raise_for_status()
+        return response.json()
 
 
-class NoAuthLMSClient(NoAuthenticationLmsApiClient):
+class NoAuthLMSClient(NoAuthAPIClient):
     """
-    Class to build an LMS client to make calls to the LMS.
+    The LMS API client to make calls to the LMS without authentication.
     """
 
     API_BASE_URL = settings.LMS_INTERNAL_ROOT_URL
@@ -560,29 +525,7 @@ class NoAuthLMSClient(NoAuthenticationLmsApiClient):
         Returns:
             dict: Response containing LMS service health.
         """
-        return self.client.heartbeat.get()
-
-
-def parse_lms_api_datetime(datetime_string, datetime_format=LMS_API_DATETIME_FORMAT):
-    """
-    Parse a received datetime into a timezone-aware, Python datetime object.
-
-    Arguments:
-        datetime_string: A string to be parsed.
-        datetime_format: A datetime format string to be used for parsing
-
-    """
-    if isinstance(datetime_string, datetime.datetime):
-        date_time = datetime_string
-    else:
-        try:
-            date_time = datetime.datetime.strptime(datetime_string, datetime_format)
-        except ValueError:
-            date_time = datetime.datetime.strptime(datetime_string, LMS_API_DATETIME_FORMAT_WITHOUT_TIMEZONE)
-
-    # If the datetime format didn't include a timezone, then set to UTC.
-    # Note that if we're using the default LMS_API_DATETIME_FORMAT, it ends in 'Z',
-    # which denotes UTC for ISO-8661.
-    if date_time.tzinfo is None:
-        date_time = date_time.replace(tzinfo=timezone.utc)
-    return date_time
+        api_url = self.get_api_url("heartbeat")
+        response = self.client.get(api_url)
+        response.raise_for_status()
+        return response.json()
