@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 These settings are here to use during tests, because django requires them.
 
@@ -6,16 +5,24 @@ In a real-world use case, apps in this project are installed into other
 Django applications, so these settings will not be used.
 """
 
+import os
 from os.path import abspath, dirname, join
 
 from celery import Celery
 
 from enterprise.constants import (
+    DEFAULT_ENTERPRISE_ENROLLMENT_INTENTIONS_ROLE,
     ENTERPRISE_ADMIN_ROLE,
     ENTERPRISE_CATALOG_ADMIN_ROLE,
     ENTERPRISE_DASHBOARD_ADMIN_ROLE,
     ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
+    ENTERPRISE_FULFILLMENT_OPERATOR_ROLE,
+    ENTERPRISE_LEARNER_ROLE,
     ENTERPRISE_OPERATOR_ROLE,
+    ENTERPRISE_SSO_ORCHESTRATOR_OPERATOR_ROLE,
+    PROVISIONING_ENTERPRISE_CUSTOMER_ADMIN_ROLE,
+    PROVISIONING_PENDING_ENTERPRISE_CUSTOMER_ADMIN_ROLE,
+    SYSTEM_ENTERPRISE_PROVISIONING_ADMIN_ROLE,
 )
 
 
@@ -34,16 +41,17 @@ def root(*args):
 
 
 DATABASES = {
-    "default": {
-        "ENGINE": "django.db.backends.sqlite3",
-        "NAME": "default.db",
-        "USER": "",
-        "PASSWORD": "",
-        "HOST": "",
-        "PORT": "",
-    }
+    'default': {
+        'ENGINE': os.environ.get('DB_ENGINE', 'django.db.backends.sqlite3'),
+        'NAME': os.environ.get('DB_NAME', 'default.db'),
+        'USER': os.environ.get('DB_USER', ''),
+        'PASSWORD': os.environ.get('DB_PASSWORD', ''),
+        'HOST': os.environ.get('DB_HOST', ''),
+        'PORT': os.environ.get('DB_PORT', ''),
+    },
 }
 
+
 INSTALLED_APPS = (
     "django.contrib.auth",
     "django.contrib.contenttypes",
@@ -59,6 +67,7 @@ INSTALLED_APPS = (
     "integrated_channels.integrated_channel",
     "integrated_channels.cornerstone",
     "integrated_channels.degreed",
+    "integrated_channels.degreed2",
     "integrated_channels.canvas",
     "integrated_channels.blackboard",
     "integrated_channels.moodle",
@@ -66,6 +75,7 @@ INSTALLED_APPS = (
     "integrated_channels.xapi",
     "edx_rbac",
     "rules.apps.AutodiscoverRulesConfig",
+    "oauth2_provider",
 )
 
 MIDDLEWARE_CLASSES = [
@@ -147,7 +157,7 @@ ENTERPRISE_SUPPORT_URL = "http://foo"
 
 ENTERPRISE_TAGLINE = "High-quality online learning opportunities from the world's best universities"
 
-OAUTH_ID_TOKEN_EXPIRATION = 60*60  # in seconds
+OAUTH_ID_TOKEN_EXPIRATION = 60 * 60  # in seconds
 
 EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'
 
@@ -155,6 +165,7 @@ DEFAULT_FROM_EMAIL = 'course_staff@example.com'
 
 USER_THROTTLE_RATE = '190/minute'
 SERVICE_USER_THROTTLE_RATE = '200/minute'
+SERVICE_USER_HIGH_THROTTLE_RATE = '200/minute'
 REST_FRAMEWORK = {
     'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
     'PAGE_SIZE': 10,
@@ -165,6 +176,7 @@ REST_FRAMEWORK = {
     'DEFAULT_THROTTLE_RATES': {
         'user': USER_THROTTLE_RATE,
         'service_user': SERVICE_USER_THROTTLE_RATE,
+        'high_service_user': SERVICE_USER_HIGH_THROTTLE_RATE,
     },
     'DATETIME_FORMAT': '%Y-%m-%dT%H:%M:%SZ',
 }
@@ -179,8 +191,9 @@ MEDIA_URL = "/"
 ENTERPRISE_ALL_SERVICE_USERNAMES = [
     'ecommerce_worker',
     'enterprise_worker',
-    'license_manager_worker',
-    'enterprise_catalog_worker',
+    'license-manager_worker',
+    'enterprise-catalog_worker',
+    'enterprise-subsidy_worker',
 ]
 
 ECOMMERCE_SERVICE_WORKER_USERNAME = 'ecommerce_worker'
@@ -196,6 +209,8 @@ COURSE_ID_PATTERN = COURSE_KEY_PATTERN.replace('course_key_string', 'course_id')
 
 USE_TZ = True
 
+TIME_ZONE = 'UTC'
+
 MKTG_URLS = {}
 
 ENTERPRISE_CUSTOMER_CATALOG_DEFAULT_CONTENT_FILTER = {
@@ -214,21 +229,31 @@ ENTERPRISE_CUSTOMER_CATALOG_DEFAULT_CONTENT_FILTER = {
     'status': 'published'
 }
 
-TABLEAU_URL = 'http://localhost'
-TABLEAU_ADMIN_USER = 'edx'
-TABLEAU_ADMIN_USER_PASSWORD = 'edx'
+SNOWFLAKE_SERVICE_USER = 'TEST@EDX.ORG'
+SNOWFLAKE_SERVICE_USER_PASSWORD = 'secret'
 
 # For testing edx-rbac rules. This is not the actual value of the setting in prod.
 SYSTEM_TO_FEATURE_ROLE_MAPPING = {
+    ENTERPRISE_LEARNER_ROLE: [
+        DEFAULT_ENTERPRISE_ENROLLMENT_INTENTIONS_ROLE,
+    ],
     ENTERPRISE_ADMIN_ROLE: [
         ENTERPRISE_DASHBOARD_ADMIN_ROLE,
         ENTERPRISE_CATALOG_ADMIN_ROLE,
-        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE
+        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
+        DEFAULT_ENTERPRISE_ENROLLMENT_INTENTIONS_ROLE,
     ],
     ENTERPRISE_OPERATOR_ROLE: [
         ENTERPRISE_DASHBOARD_ADMIN_ROLE,
         ENTERPRISE_CATALOG_ADMIN_ROLE,
-        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE
+        ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE,
+        ENTERPRISE_FULFILLMENT_OPERATOR_ROLE,
+        ENTERPRISE_SSO_ORCHESTRATOR_OPERATOR_ROLE,
+        DEFAULT_ENTERPRISE_ENROLLMENT_INTENTIONS_ROLE,
+    ],
+    SYSTEM_ENTERPRISE_PROVISIONING_ADMIN_ROLE: [
+        PROVISIONING_ENTERPRISE_CUSTOMER_ADMIN_ROLE,
+        PROVISIONING_PENDING_ENTERPRISE_CUSTOMER_ADMIN_ROLE,
     ],
 }
 
@@ -276,7 +301,7 @@ EVENT_TRACKING_PROCESSORS = []
 
 #################################### CELERY ####################################
 
-app = Celery('enterprise')  # pylint: disable=invalid-name
+app = Celery('enterprise')
 app.conf.task_protocol = 1
 app.config_from_object('django.conf:settings')
 
@@ -318,4 +343,54 @@ INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT = {
     'SAP': 1,
 }
 
-LANGUAGE_COOKIE = 'openedx-language-preference'
+LANGUAGE_COOKIE_NAME = "openedx-language-preference"
+SHARED_COOKIE_DOMAIN = ''
+
+ENTERPRISE_BACKEND_SERVICE_EDX_OAUTH2_PROVIDER_URL = f'{LMS_INTERNAL_ROOT_URL}/oauth2'
+ENTERPRISE_BACKEND_SERVICE_EDX_OAUTH2_KEY = 'test_backend_oauth2_key'
+ENTERPRISE_BACKEND_SERVICE_EDX_OAUTH2_SECRET = 'test_backend_oauth2_secret'
+
+ECOMMERCE_API_URL = 'https://ecommerce.example.com/api/v2/'
+
+LOGIN_REDIRECT_WHITELIST = [
+    'success.url',
+    'failure.url',
+    'google.com',
+    'facebook.com'
+]
+
+ENTERPRISE_MANUAL_REPORTING_CUSTOMER_UUIDS = ['12aacfee8ffa4cb3bed1059565a57f06',]
+EXEC_ED_LANDING_PAGE = 'https://www.edx-external.com/account'
+
+
+# disable indexing on history_date. Otherwise it will add new alter migrations.
+SIMPLE_HISTORY_DATE_INDEX = False
+
+CHAT_COMPLETION_API_V2 = 'https://example.com/chat/completion'
+ENTERPRISE_ANALYSIS_CLIENT_ID = 'test_client_id'
+ENTERPRISE_ANALYSIS_SYSTEM_PROMPT = 'This is a test prompt'
+
+LEARNER_ENGAGEMENT_PROMPT_FOR_ACTIVE_CONTRACT = 'Create learner engagement report for active contracts.'
+LEARNER_ENGAGEMENT_PROMPT_FOR_NON_ACTIVE_CONTRACT = 'Create learner engagement report for non active contracts.'
+LEARNER_PROGRESS_PROMPT_FOR_ACTIVE_CONTRACT = 'Create learner progress report for active contracts.'
+LEARNER_PROGRESS_PROMPT_FOR_NON_ACTIVE_CONTRACT = 'Create learner progress report for non active contracts.'
+
+ENTERPRISE_SSO_ORCHESTRATOR_WORKER_USERNAME = 'username'
+ENTERPRISE_SSO_ORCHESTRATOR_WORKER_PASSWORD = 'password'
+ENTERPRISE_SSO_ORCHESTRATOR_BASE_URL = 'https://foobar.com'
+ENTERPRISE_SSO_ORCHESTRATOR_CONFIGURE_PATH = 'configure'
+ENTERPRISE_SSO_ORCHESTRATOR_CONFIGURE_EDX_OAUTH_PATH = 'configure-edx-oauth'
+
+ENTERPRISE_BRAZE_API_KEY = 'test-api-key'
+EDX_BRAZE_API_SERVER = 'test-api-server'
+BRAZE_GROUPS_INVITATION_EMAIL_CAMPAIGN_ID = 'test-invitation-campaign-id'
+BRAZE_GROUPS_REMOVAL_EMAIL_CAMPAIGN_ID = 'test-removal-campaign-id'
+
+STORAGES = {
+    'default': {
+        'BACKEND': 'django.core.files.storage.FileSystemStorage',
+    },
+    'staticfiles': {
+        'BACKEND': 'django.contrib.staticfiles.storage.StaticFilesStorage',
+    },
+}
