@@ -4,9 +4,10 @@
 
 from rest_framework import routers
 
-from .views import BlackboardConfigurationViewSet
+from .views import BlackboardConfigurationViewSet, BlackboardGlobalConfigurationViewSet
 
 app_name = 'blackboard'
-router = routers.DefaultRouter()  # pylint: disable=invalid-name
+router = routers.DefaultRouter()
 router.register(r'configuration', BlackboardConfigurationViewSet, basename="configuration")
+router.register(r'global-configuration', BlackboardGlobalConfigurationViewSet, basename="global-configuration")
 urlpatterns = router.urls
