@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 URL definitions for integrated_channels/canvas API version 1.
 """
@@ -8,6 +7,6 @@ from rest_framework import routers
 from .views import CanvasConfigurationViewSet
 
 app_name = 'canvas'
-router = routers.DefaultRouter()  # pylint: disable=invalid-name
+router = routers.DefaultRouter()
 router.register(r'configuration', CanvasConfigurationViewSet, basename="configuration")
 urlpatterns = router.urls
