@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Client for connecting to Blackboard.
 """
@@ -6,17 +5,19 @@ import base64
 import copy
 import json
 import logging
+import time
 from http import HTTPStatus
+from urllib.parse import urljoin
 
 import requests
-from six.moves.urllib.parse import urljoin  # pylint: disable=import-error
 
 from django.apps import apps
+from django.db import transaction
 
 from integrated_channels.blackboard.exporters.content_metadata import BLACKBOARD_COURSE_CONTENT_NAME
 from integrated_channels.exceptions import ClientError
 from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
-from integrated_channels.utils import refresh_session_if_expired
+from integrated_channels.utils import generate_formatted_log, refresh_session_if_expired, stringify_and_store_api_record
 
 LOGGER = logging.getLogger(__name__)
 
@@ -25,12 +26,15 @@ GRADEBOOK_PATH = '/learn/api/public/v1/courses/{course_id}/gradebook/columns'
 ENROLLMENT_PATH = '/learn/api/public/v1/courses/{course_id}/users'
 COURSE_PATH = '/learn/api/public/v1/courses'
 POST_GRADE_COLUMN_PATH = '/learn/api/public/v2/courses/{course_id}/gradebook/columns'
+PATCH_GRADE_COLUMN_PATH = '/learn/api/public/v2/courses/{course_id}/gradebook/columns/{column_id}'
 POST_GRADE_PATH = '/learn/api/public/v2/courses/{course_id}/gradebook/columns/{column_id}/users/{user_id}'
 COURSE_V3_PATH = '/learn/api/public/v3/courses/{course_id}'
 COURSES_V3_PATH = '/learn/api/public/v3/courses'
 COURSE_CONTENT_PATH = '/learn/api/public/v1/courses/{course_id}/contents'
 COURSE_CONTENT_CHILDREN_PATH = '/learn/api/public/v1/courses/{course_id}/contents/{content_id}/children'
 COURSE_CONTENT_DELETE_PATH = '/learn/api/public/v1/courses/{course_id}/contents/{content_id}'
+GRADEBOOK_COLUMN_DESC = "edX learner's grade"
+PAGE_TRAVERSAL_LIMIT = 250
 
 
 class BlackboardAPIClient(IntegratedChannelApiClient):
@@ -47,6 +51,11 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             configuration model for connecting with Blackboard
         """
         super().__init__(enterprise_configuration)
+        BlackboardGlobalConfiguration = apps.get_model(
+            'blackboard',
+            'BlackboardGlobalConfiguration'
+        )
+        self.global_blackboard_config = BlackboardGlobalConfiguration.current()
         self.config = apps.get_app_config('blackboard')
         self.session = None
         self.expires_at = None
@@ -68,10 +77,25 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         copy_of_channel_metadata = copy.deepcopy(channel_metadata_item)
         copy_of_channel_metadata['course_metadata']['courseId'] = course_id_generated
 
-        LOGGER.info("Creating course with courseId: %s", external_id)
         self._create_session()
         create_url = self.generate_course_create_url()
-        response = self._post(create_url, copy_of_channel_metadata['course_metadata'])
+        try:
+            response = self._post(create_url, copy_of_channel_metadata['course_metadata'])
+        except ClientError as error:
+            if error.status_code == 409 and 'Unique ID conflicts' in error.message:
+                # course already exists!
+                msg_body = (f"Course already exists with course_id {external_id},"
+                            f" and generated course_id: {course_id_generated}, not attempting creation")
+                LOGGER.warning(generate_formatted_log(
+                    self.enterprise_configuration.channel_code(),
+                    self.enterprise_configuration.enterprise_customer.uuid,
+                    None,
+                    external_id,
+                    msg_body,
+                ))
+                return HTTPStatus.NOT_MODIFIED.value, msg_body
+            else:
+                raise error
 
         # We wrap error handling in the post, but sanity check for the ID
         bb_course_id = response.json().get('id')
@@ -81,7 +105,6 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
                 HTTPStatus.NOT_FOUND.value
             )
 
-        LOGGER.info("Creating content page for Blackboard course with course ID={}".format(external_id))
         course_created_response = self.create_integration_content_for_course(bb_course_id, copy_of_channel_metadata)
 
         success_body = 'Successfully created Blackboard integration course={bb_course_id} with integration ' \
@@ -102,7 +125,6 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         course_id = self._resolve_blackboard_course_id(external_id)
         BlackboardAPIClient._validate_course_id(course_id, external_id)
 
-        LOGGER.info('Updating course with courseId: {}'.format(course_id))
         update_url = self.generate_course_update_url(course_id)
         response = self._patch(update_url, channel_metadata_item.get('course_metadata'))
 
@@ -125,7 +147,15 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
                 bb_content_id = content.get('id')
 
         if not bb_content_id:
-            LOGGER.info('Blackboard integrated course content not found. Generating content.')
+            LOGGER.info(
+                generate_formatted_log(
+                    self.enterprise_configuration.channel_code(),
+                    self.enterprise_configuration.enterprise_customer.uuid,
+                    None,
+                    course_id,
+                    'Blackboard integrated course content not found. Generating content.'
+                )
+            )
         else:
             course_content_delete_url = self.generate_course_content_delete_url(bb_course_id, bb_content_id)
             self._delete(course_content_delete_url)
@@ -145,9 +175,19 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         BlackboardAPIClient._validate_channel_metadata(channel_metadata_item)
         external_id = channel_metadata_item.get('externalId')
         course_id = self._resolve_blackboard_course_id(external_id)
-        BlackboardAPIClient._validate_course_id(course_id, external_id)
 
-        LOGGER.info("Deleting course with courseId: %s", course_id)
+        if not course_id:
+            return HTTPStatus.OK.value, 'Course:{} already removed.'.format(external_id)
+
+        LOGGER.info(
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                None,
+                course_id,
+                f'Deleting course with courseId: {course_id}'
+            )
+        )
         update_url = self.generate_course_update_url(course_id)
         response = self._delete(update_url)
         return response.status_code, response.text
@@ -215,7 +255,8 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         grade_column_id = self._get_or_create_integrated_grade_column(
             course_id,
             "(edX Integration) Final Grade",
-            "edx_final_grade"
+            "edx_final_grade",
+            include_in_calculations=True,
         )
 
         grade = learner_data.get('grade') * 100
@@ -228,7 +269,7 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         )
         return submission_response.status_code, success_body
 
-    def delete_course_completion(self, user_id, payload):  # pylint: disable=unused-argument
+    def delete_course_completion(self, user_id, payload):
         """TODO: course completion deletion is currently not easily supported"""
 
     @staticmethod
@@ -275,9 +316,29 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             self.expires_at,
         )
 
+    def _formatted_message(self, msg):
+        return generate_formatted_log(
+            self.enterprise_configuration.channel_code(),
+            self.enterprise_configuration.enterprise_customer.uuid,
+            None,
+            None,
+            msg,
+        )
+
+    def _log_info(self, msg):
+        LOGGER.info(self._formatted_message(msg))
+
+    def _log_error(self, msg):
+        LOGGER.error(self._formatted_message(msg))
+
     def _get_oauth_access_token(self):
         """Fetch access token using refresh_token workflow from Blackboard
 
+        Using atomic block for refresh token handling code, because we need to use the
+        most recently obtained refresh token always
+        Since any time we use a refresh token to get a new one, the prior one is invalidated
+        and we MUST use the new one for the next request.
+
         Returns:
             access_token (str): the OAuth access token to access the Blackboard server
             expires_in (int): the number of seconds after which token will expire
@@ -286,12 +347,6 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             ClientError: If an unexpected response format was received that we could not parse.
         """
 
-        if not self.enterprise_configuration.refresh_token:
-            raise ClientError(
-                "Failed to generate oauth access token: Refresh token required.",
-                HTTPStatus.INTERNAL_SERVER_ERROR.value
-            )
-
         if (not self.enterprise_configuration.blackboard_base_url
                 or not self.config.oauth_token_auth_path):
             raise ClientError(
@@ -303,50 +358,92 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             self.config.oauth_token_auth_path,
         )
 
-        auth_token_params = {
-            'grant_type': 'refresh_token',
-            'refresh_token': self.enterprise_configuration.refresh_token,
-        }
+        # Refresh token handling atomic block
+        # DO NOT USE self.enterprise_config in this block!
+        # Here we don't use the config passed from the constructor directly
+        # instead we fetch a transaction safe instance using select_for_update
+        # so that multiple sessions such as this
+        # don't interfere with each other's Blackboard refresh token state. Blackboard only allows
+        # only valid refresh token at a time, and once it's used it's invalidated and needs to be
+        # replaced by a new token
+        with transaction.atomic():
+            channel_config = apps.get_model(
+                'blackboard',
+                'BlackboardEnterpriseCustomerConfiguration'
+            ).objects.select_for_update().get(pk=self.enterprise_configuration.pk)
+            if not channel_config.refresh_token:
+                raise ClientError(
+                    "Failed to generate oauth access token: Refresh token required.",
+                    HTTPStatus.INTERNAL_SERVER_ERROR.value
+                )
 
-        auth_response = requests.post(
-            auth_token_url,
-            auth_token_params,
-            headers={
-                'Authorization': self._create_auth_header(),
-                'Content-Type': 'application/x-www-form-urlencoded'
+            auth_token_params = {
+                'grant_type': 'refresh_token',
+                'refresh_token': channel_config.refresh_token,
             }
-        )
-        if auth_response.status_code >= 400:
-            raise ClientError(auth_response.text, auth_response.status_code)
-        try:
-            data = auth_response.json()
-            # do not forget to save the new refresh token otherwise subsequent requests will fail
-            self.enterprise_configuration.refresh_token = data["refresh_token"]
-            self.enterprise_configuration.save()
-            return data['access_token'], data["expires_in"]
-        except (KeyError, ValueError) as error:
-            raise ClientError(auth_response.text, auth_response.status_code) from error
+
+            auth_response = requests.post(
+                auth_token_url,
+                auth_token_params,
+                headers={
+                    'Authorization': self._create_auth_header(),
+                    'Content-Type': 'application/x-www-form-urlencoded'
+                }
+            )
+            if auth_response.status_code >= 400:
+                raise ClientError(
+                    f"BLACKBOARD: Access/Refresh token fetch failure, "
+                    f"enterprise_customer_uuid: {channel_config.enterprise_customer.uuid}, "
+                    f"blackboard_base_url: {channel_config.blackboard_base_url}, "
+                    f"auth_response_text: {auth_response.text}"
+                    f"config_last_modified: {channel_config.modified}",
+                    auth_response.status_code,
+                )
+            try:
+                data = auth_response.json()
+                # do not forget to save the new refresh token otherwise subsequent requests will fail
+                if "refresh_token" not in data:
+                    self._log_info("Server did not return refresh_token, keeping existing one")
+                else:
+                    # refresh token was returned by server, needs to be used
+                    fetched_refresh_token = data["refresh_token"]
+                    if not fetched_refresh_token.strip():
+                        # we are out of luck, can't use this invalid token
+                        # and we probably can't get unstuck without customer re-doing oauth url workflow
+                        self._log_error("Fetched a new refresh token, but it was empty, not using it!")
+                    else:
+                        channel_config.refresh_token = fetched_refresh_token
+                        self._log_info("Fetched a new refresh token, replacing current one")
+                        channel_config.save()
+                        # We do not want any fail-prone code in this atomic block after this line
+                        # it's because if something else fails, it will roll back the just-saved
+                        # refresh token!
+                return data['access_token'], data["expires_in"]
+            except (KeyError, ValueError) as error:
+                raise ClientError(auth_response.text, auth_response.status_code) from error
 
     def _create_auth_header(self):
         """
         auth header in oauth2 token format as required by blackboard doc
         """
-        if not self.enterprise_configuration.client_id:
-            raise ClientError(
-                "Failed to generate oauth access token: Client ID required.",
-                HTTPStatus.INTERNAL_SERVER_ERROR.value
-            )
-        if not self.enterprise_configuration.client_secret:
-            raise ClientError(
-                "Failed to generate oauth access token: Client secret required.",
-                HTTPStatus.INTERNAL_SERVER_ERROR.value
-            )
-        return 'Basic {}'.format(
-            base64.b64encode(u'{key}:{secret}'.format(
-                key=self.enterprise_configuration.client_id,
-                secret=self.enterprise_configuration.client_secret
-            ).encode('utf-8')).decode()
-        )
+        app_key = self.enterprise_configuration.decrypted_client_id
+        if not app_key:
+            if not self.global_blackboard_config.app_key:
+                raise ClientError(
+                    "Failed to generate oauth access token: Client ID required.",
+                    HTTPStatus.INTERNAL_SERVER_ERROR.value
+                )
+            app_key = self.global_blackboard_config.app_key
+
+        app_secret = self.enterprise_configuration.decrypted_client_secret
+        if not app_secret:
+            if not self.global_blackboard_config.app_secret:
+                raise ClientError(
+                    "Failed to generate oauth access token: Client secret required.",
+                    HTTPStatus.INTERNAL_SERVER_ERROR.value
+                )
+            app_secret = self.global_blackboard_config.app_secret
+        return f"Basic {base64.b64encode(f'{app_key}:{app_secret}'.encode('utf-8')).decode()}"
 
     def generate_blackboard_course_id(self, external_id):
         """
@@ -354,6 +451,35 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         """
         return str(abs(hash(external_id)))
 
+    def generate_blackboard_gradebook_column_data(self, external_id, grade_column_name, points_possible,
+                                                  include_in_calculations=False):
+        """
+        Properly formatted json data to create a new gradebook column in a blackboard course
+
+        Note: Potential customization here per-customer, if the need arises.
+        """
+        return {
+            "externalId": external_id,
+            "name": grade_column_name,
+            "displayName": grade_column_name,
+            "description": GRADEBOOK_COLUMN_DESC,
+            "externalGrade": False,
+            "score": {
+                "possible": points_possible
+            },
+            "availability": {
+                "available": "Yes"
+            },
+            "grading": {
+                "type": "Manual",
+                "scoringModel": "Last",
+                "anonymousGrading": {
+                    "type": "None",
+                }
+            },
+            "includeInCalculations": include_in_calculations,
+        }
+
     def generate_gradebook_url(self, course_id):
         """
         Blackboard API url helper method.
@@ -413,6 +539,19 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             path=POST_GRADE_COLUMN_PATH.format(course_id=course_id),
         )
 
+    def generate_update_grade_column_url(self, course_id, column_id):
+        """
+        Blackboard API url helper method.
+        Path: Update course grade column
+        """
+        return '{base}{path}'.format(
+            base=self.enterprise_configuration.blackboard_base_url,
+            path=PATCH_GRADE_COLUMN_PATH.format(
+                course_id=course_id,
+                column_id=column_id,
+            ),
+        )
+
     def generate_post_users_grade_url(self, course_id, column_id, user_id):
         """
         Blackboard API url helper method.
@@ -461,7 +600,19 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         """
         Returns request's get response and raises Client Errors if appropriate.
         """
+        start_time = time.time()
         get_response = self.session.get(url, params=data)
+        time_taken = time.time() - start_time
+        stringify_and_store_api_record(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=url,
+            data=data,
+            time_taken=time_taken,
+            status_code=get_response.status_code,
+            response_body=get_response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
         if get_response.status_code >= 400:
             raise ClientError(get_response.text, get_response.status_code)
         return get_response
@@ -470,7 +621,19 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         """
         Returns request's patch response and raises Client Errors if appropriate.
         """
+        start_time = time.time()
         patch_response = self.session.patch(url, json=data)
+        time_taken = time.time() - start_time
+        stringify_and_store_api_record(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=url,
+            data=data,
+            time_taken=time_taken,
+            status_code=patch_response.status_code,
+            response_body=patch_response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
         if patch_response.status_code >= 400:
             raise ClientError(patch_response.text, patch_response.status_code)
         return patch_response
@@ -479,7 +642,20 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         """
         Returns request's post response and raises Client Errors if appropriate.
         """
+        start_time = time.time()
         post_response = self.session.post(url, json=data)
+        time_taken = time.time() - start_time
+        stringify_and_store_api_record(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=url,
+            data=data,
+            time_taken=time_taken,
+            status_code=post_response.status_code,
+            response_body=post_response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
+
         if post_response.status_code >= 400:
             raise ClientError(post_response.text, post_response.status_code)
         return post_response
@@ -488,7 +664,19 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
         """
         Returns request's delete response and raises Client Errors if appropriate.
         """
+        start_time = time.time()
         response = self.session.delete(url)
+        time_taken = time.time() - start_time
+        stringify_and_store_api_record(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=url,
+            data='',
+            time_taken=time_taken,
+            status_code=response.status_code,
+            response_body=response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
         if response.status_code >= 400:
             raise ClientError(response.text, response.status_code)
         return response
@@ -515,7 +703,8 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             HTTPStatus.NOT_FOUND.value
         )
 
-    def _get_or_create_integrated_grade_column(self, bb_course_id, grade_column_name, external_id, points_possible=100):
+    def _get_or_create_integrated_grade_column(self, bb_course_id, grade_column_name, external_id, points_possible=100,
+                                               include_in_calculations=False):
         """
         Helper method to search an edX integrated Blackboard course for the designated edX grade column.
         If the column does not yet exist within the course, create it.
@@ -525,37 +714,69 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             bb_course_id (str): The Blackboard course ID in which to search for the edX final grade,
             grade column.
         """
-        grade_column_response = self._get(self.generate_gradebook_url(bb_course_id))
-        parsed_response = grade_column_response.json()
-        grade_columns = parsed_response.get('results')
-
+        gradebook_column_url = self.generate_gradebook_url(bb_course_id)
         grade_column_id = None
-        for grade_column in grade_columns:
-            if grade_column.get('externalId') == external_id:
-                grade_column_id = grade_column.get('id')
-        if not grade_column_id:
-            # Potential customization here per-customer, if the need arises.
-            grade_column_data = {
-                "externalId": external_id,
-                "name": grade_column_name,
-                "displayName": grade_column_name,
-                "description": "edX learner's grade.",
-                "externalGrade": False,
-                "score": {
-                    "possible": points_possible
-                },
-                "availability": {
-                    "available": "Yes"
-                },
-                "grading": {
-                    "type": "Manual",
-                    "scoringModel": "Last",
-                    "anonymousGrading": {
-                        "type": "None",
-                    }
-                },
-            }
+        more_pages_present = True
+        current_page_count = 0
 
+        # Page count of 250 is a preventative of infinite while loops
+        while more_pages_present and current_page_count <= PAGE_TRAVERSAL_LIMIT:
+            grade_column_response = self._get(gradebook_column_url)
+            parsed_response = grade_column_response.json()
+            grade_columns = parsed_response.get('results')
+
+            for grade_column in grade_columns:
+                if grade_column.get('externalId') == external_id:
+                    grade_column_id = grade_column.get('id')
+                    # if includeInCalculations has legacy value, correct it
+                    if (
+                        grade_column.get('includeInCalculations') and
+                        grade_column.get('includeInCalculations') != include_in_calculations
+                    ):
+                        calculations_data = {"includeInCalculations": include_in_calculations}
+                        self._patch(
+                            self.generate_update_grade_column_url(bb_course_id, grade_column_id),
+                            calculations_data
+                        )
+                    break
+            # Blackboard's pagination is returned within the response json if it exists
+            # Example:
+            # Response = {
+            #     'results': [{
+            #         'id': 1,
+            #         ...
+            #     }, {
+            #         'id', 2,
+            #         ...
+            #     }],
+            #     'paging': {
+            #         'nextPage': '/learn/api/public/v1/courses/<courseID>/gradebook/columns?offset=200'
+            #     }
+            # }
+            if parsed_response.get('paging') and not grade_column_id:
+                gradebook_column_url = '{}{}'.format(
+                    self.enterprise_configuration.blackboard_base_url,
+                    parsed_response.get('paging').get('nextPage'),
+                )
+                current_page_count += 1
+            else:
+                more_pages_present = False
+
+        if current_page_count == PAGE_TRAVERSAL_LIMIT:
+            LOGGER.warning(
+                generate_formatted_log(
+                    self.enterprise_configuration.channel_code(),
+                    self.enterprise_configuration.enterprise_customer.uuid,
+                    None,
+                    None,
+                    f'Max page limit hit while traversing blackboard API for course={external_id}'
+                )
+            )
+
+        if not grade_column_id:
+            grade_column_data = self.generate_blackboard_gradebook_column_data(
+                external_id, grade_column_name, points_possible, include_in_calculations
+            )
             response = self._post(self.generate_create_grade_column_url(bb_course_id), grade_column_data)
             parsed_response = response.json()
             grade_column_id = parsed_response.get('id')
@@ -732,3 +953,17 @@ class BlackboardAPIClient(IntegratedChannelApiClient):
             )
 
         return course_fully_created_response
+
+    def cleanup_duplicate_assignment_records(self, courses):
+        """
+        Not implemented yet.
+        """
+        LOGGER.error(
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                None,
+                None,
+                "Blackboard integrated channel does not yet support assignment deduplication."
+            )
+        )
