@@ -1,18 +1,21 @@
-# -*- coding: utf-8 -*-
 """
 Client for connecting to Degreed.
 """
 
 import datetime
+import logging
 import time
+from urllib.parse import urljoin
 
 import requests
-from six.moves.urllib.parse import urljoin  # pylint: disable=import-error
 
 from django.apps import apps
 
 from integrated_channels.exceptions import ClientError
 from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
+from integrated_channels.utils import generate_formatted_log
+
+LOGGER = logging.getLogger(__name__)
 
 
 class DegreedAPIClient(IntegratedChannelApiClient):
@@ -44,8 +47,31 @@ class DegreedAPIClient(IntegratedChannelApiClient):
         """
         Not implemented yet.
         """
+        LOGGER.error(
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                None,
+                None,
+                "Degreed integrated channel does not yet support assessment reporting."
+            )
+        )
 
-    def create_course_completion(self, user_id, payload):  # pylint: disable=unused-argument
+    def cleanup_duplicate_assignment_records(self, courses):
+        """
+        Not implemented yet.
+        """
+        LOGGER.error(
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                None,
+                None,
+                "Degreed integrated channel does not yet support assignment deduplication."
+            )
+        )
+
+    def create_course_completion(self, user_id, payload):
         """
         Send a completion status payload to the Degreed Completion Status endpoint
 
@@ -68,7 +94,7 @@ class DegreedAPIClient(IntegratedChannelApiClient):
             self.COMPLETION_PROVIDER_SCOPE
         )
 
-    def delete_course_completion(self, user_id, payload):  # pylint: disable=unused-argument
+    def delete_course_completion(self, user_id, payload):
         """
         Delete a completion status previously sent to the Degreed Completion Status endpoint
 
