@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Database models for Enterprise Integrated Channel Degreed.
 """
@@ -7,22 +6,22 @@ import json
 from logging import getLogger
 
 from config_models.models import ConfigurationModel
-from simple_history.models import HistoricalRecords
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 from integrated_channels.degreed.exporters.content_metadata import DegreedContentMetadataExporter
 from integrated_channels.degreed.exporters.learner_data import DegreedLearnerExporter
 from integrated_channels.degreed.transmitters.content_metadata import DegreedContentMetadataTransmitter
 from integrated_channels.degreed.transmitters.learner_data import DegreedLearnerTransmitter
-from integrated_channels.integrated_channel.models import EnterpriseCustomerPluginConfiguration
+from integrated_channels.integrated_channel.models import (
+    EnterpriseCustomerPluginConfiguration,
+    LearnerDataTransmissionAudit,
+)
+from integrated_channels.utils import is_valid_url
 
 LOGGER = getLogger(__name__)
 
 
-# pylint: disable=feature-toggle-needs-doc
-@python_2_unicode_compatible
 class DegreedGlobalConfiguration(ConfigurationModel):
     """
     The global configuration for integrating with Degreed.
@@ -67,7 +66,6 @@ class DegreedGlobalConfiguration(ConfigurationModel):
         return self.__str__()
 
 
-@python_2_unicode_compatible
 class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfiguration):
     """
     The Enterprise-specific configuration we need for integrating with Degreed.
@@ -77,6 +75,8 @@ class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfigurati
 
     key = models.CharField(
         max_length=255,
+        blank=True,
+        default='',
         verbose_name="API Client ID",
         help_text=(
             "The API Client ID provided to edX by the enterprise customer to be used to make API "
@@ -86,6 +86,8 @@ class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfigurati
 
     secret = models.CharField(
         max_length=255,
+        blank=True,
+        default='',
         verbose_name="API Client Secret",
         help_text=(
             "The API Client Secret provided to edX by the enterprise customer to be used to make API "
@@ -95,18 +97,24 @@ class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfigurati
 
     degreed_company_id = models.CharField(
         max_length=255,
+        blank=True,
+        default='',
         verbose_name="Degreed Organization Code",
         help_text="The organization code provided to the enterprise customer by Degreed."
     )
 
     degreed_base_url = models.CharField(
         max_length=255,
+        blank=True,
+        default='',
         verbose_name="Degreed Base URL",
         help_text="The base URL used for API requests to Degreed, i.e. https://degreed.com."
     )
 
     degreed_user_id = models.CharField(
         max_length=255,
+        blank=True,
+        default='',
         verbose_name="Degreed User ID",
         help_text=(
             "The Degreed User ID provided to the content provider by Degreed. "
@@ -116,6 +124,8 @@ class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfigurati
 
     degreed_user_password = models.CharField(
         max_length=255,
+        blank=True,
+        default='',
         verbose_name="Degreed User Password",
         help_text=(
             "The Degreed User Password provided to the content provider by Degreed. "
@@ -130,11 +140,40 @@ class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfigurati
         help_text="The provider code that Degreed gives to the content provider."
     )
 
-    history = HistoricalRecords()
-
     class Meta:
         app_label = 'degreed'
 
+    @property
+    def is_valid(self):
+        """
+        Returns whether or not the configuration is valid and ready to be activated
+
+        Args:
+            obj: The instance of DegreedEnterpriseCustomerConfiguration
+                being rendered with this admin form.
+        """
+        missing_items = {'missing': []}
+        incorrect_items = {'incorrect': []}
+        if not self.key:
+            missing_items.get('missing').append('key')
+        if not self.secret:
+            missing_items.get('missing').append('secret')
+        if not self.degreed_company_id:
+            missing_items.get('missing').append('degreed_company_id')
+        if not self.degreed_base_url:
+            missing_items.get('missing').append('degreed_base_url')
+        if not self.degreed_user_id:
+            missing_items.get('missing').append('degreed_user_id')
+        if not self.degreed_user_password:
+            missing_items.get('missing').append('degreed_user_password')
+        if not self.provider_id:
+            missing_items.get('missing').append('provider_id')
+        if not is_valid_url(self.degreed_base_url):
+            incorrect_items.get('incorrect').append('degreed_base_url')
+        if len(self.display_name) > 20:
+            incorrect_items.get('incorrect').append('display_name')
+        return missing_items, incorrect_items
+
     def __str__(self):
         """
         Return human-readable string representation.
@@ -181,12 +220,11 @@ class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfigurati
         return DegreedContentMetadataExporter(user, self)
 
 
-@python_2_unicode_compatible
-class DegreedLearnerDataTransmissionAudit(models.Model):
+class DegreedLearnerDataTransmissionAudit(LearnerDataTransmissionAudit):
     """
     The payload we sent to Degreed at a given point in time for an enterprise course enrollment.
 
-    .. pii: The degreed_user_email model field contains PII.
+    .. pii: The user_email AND degreed_user_email model fields contain PII.
     .. pii_types: email_address
     .. pii_retirement: consumer_api
     """
@@ -197,39 +235,24 @@ class DegreedLearnerDataTransmissionAudit(models.Model):
         null=False
     )
 
-    enterprise_course_enrollment_id = models.PositiveIntegerField(
-        blank=False,
-        null=False,
-        db_index=True
-    )
-
-    course_id = models.CharField(
-        max_length=255,
-        blank=False,
-        null=False,
-        help_text="The course run's key which is used to uniquely identify the course for Degreed."
-    )
-
-    course_completed = models.BooleanField(
-        default=True,
-        help_text="The learner's course completion status transmitted to Degreed."
-    )
-
-    completed_timestamp = models.CharField(
+    degreed_completed_timestamp = models.CharField(
         max_length=10,
+        blank=True,
+        null=True,
         help_text=(
             'Represents the Degreed representation of a timestamp: yyyy-mm-dd, '
             'which is always 10 characters.'
         )
     )
 
-    # Request-related information.
-    status = models.CharField(max_length=100, blank=False, null=False)
-    error_message = models.TextField(blank=True)
-    created = models.DateTimeField(auto_now_add=True)
-
     class Meta:
         app_label = 'degreed'
+        indexes = [
+            models.Index(
+                fields=['enterprise_customer_uuid', 'plugin_configuration_id'],
+                name="degreed_customer_plugin_idx"
+            ),
+        ]
 
     def __str__(self):
         """
@@ -251,7 +274,7 @@ class DegreedLearnerDataTransmissionAudit(models.Model):
         """
         return self.__str__()
 
-    def serialize(self, *args, **kwargs):  # pylint: disable=unused-argument
+    def serialize(self, *args, **kwargs):
         """
         Return a JSON-serialized representation.
 
@@ -266,17 +289,12 @@ class DegreedLearnerDataTransmissionAudit(models.Model):
         enterprise_configuration = kwargs.get('enterprise_configuration')
         degreed_company_id = enterprise_configuration.degreed_company_id \
             if hasattr(enterprise_configuration, 'degreed_company_id') else ''
-        return json.dumps(self._payload_data(degreed_company_id), sort_keys=True)
-
-    def _payload_data(self, degreed_company_id):
-        """
-        Convert the audit record's fields into Degreed key/value pairs.
-        """
-        return {
+        json_payload = {
             'orgCode': degreed_company_id,
             'completions': [{
                 'email': self.degreed_user_email,
                 'id': self.course_id,
-                'completionDate': self.completed_timestamp,
+                'completionDate': self.degreed_completed_timestamp,
             }]
         }
+        return json.dumps(json_payload, sort_keys=True)
