@@ -1,8 +1,18 @@
-# -*- coding: utf-8 -*-
 """
 Base API client for integrated channels.
 """
 
+from enum import Enum
+
+
+class IntegratedChannelHealthStatus(Enum):
+    """
+    Health status list for Integrated Channels
+    """
+    HEALTHY = 'HEALTHY'
+    INVALID_CONFIG = 'INVALID_CONFIG'
+    CONNECTION_FAILURE = 'CONNECTION_FAILURE'
+
 
 class IntegratedChannelApiClient:
     """
@@ -66,3 +76,26 @@ class IntegratedChannelApiClient:
         Send a request to the integrated channel's grade API to update the assessment level reporting status for a user.
         """
         raise NotImplementedError()
+
+    def cleanup_duplicate_assignment_records(self, courses):
+        """
+        Delete duplicate assignments transmitted through the integrated channel's API.
+        """
+        raise NotImplementedError()
+
+    def health_check(self):
+        """Check integrated channel's config health
+
+        Returns: IntegratedChannelHealthStatus
+            HEALTHY if configuration is valid
+            INVALID_CONFIG if configuration is incomplete/invalid
+        """
+        is_valid = self.enterprise_configuration.is_valid
+        missing_fields = is_valid[0]
+        missing_ct = len(missing_fields['missing']) if 'missing' in missing_fields else 0
+        incorrect_fields = is_valid[1]
+        incorrect_ct = len(incorrect_fields['incorrect']) if 'incorrect' in incorrect_fields else 0
+        if missing_ct > 0 or incorrect_ct > 0:
+            return IntegratedChannelHealthStatus.INVALID_CONFIG
+        else:
+            return IntegratedChannelHealthStatus.HEALTHY
