@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Transmits information about an enterprise's course catalog to connected IntegratedChannels
 """
@@ -38,6 +37,7 @@ class Command(IntegratedChannelCommandMixin, BaseCommand):
         Transmit the courseware data for the EnterpriseCustomer(s) to the active integration channels.
         """
         username = options['catalog_user']
+        options['content_metadata_job_support'] = True
 
         # Before we do a whole bunch of database queries, make sure that the user we were passed exists.
         try:
@@ -50,4 +50,5 @@ class Command(IntegratedChannelCommandMixin, BaseCommand):
         for channel in channels:
             channel_code = channel.channel_code()
             channel_pk = channel.pk
-            transmit_content_metadata.delay(username, channel_code, channel_pk)
+            # NOTE pass arguments as named kwargs for use in lock key
+            transmit_content_metadata.delay(username=username, channel_code=channel_code, channel_pk=channel_pk)
