@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Package for generic data transmitters which send serialized data to integrated channels.
 """
@@ -25,7 +24,7 @@ class Transmitter:
         self.enterprise_configuration = enterprise_configuration
         self.client = client(enterprise_configuration) if client else None
 
-    def transmit(self, payload, **kwargs):
+    def transmit(self, create_payload, update_payload, delete_payload):
         """
         The abstract interface method for sending exported data to an integrated channel through its API client.
         """
