@@ -1,9 +1,10 @@
-# -*- coding: utf-8 -*-
 """
 Client for connecting to Moodle.
 """
 
 import json
+import logging
+import time
 from http import HTTPStatus
 from urllib.parse import urlencode, urljoin
 
@@ -13,8 +14,33 @@ from django.apps import apps
 
 from integrated_channels.exceptions import ClientError
 from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
+from integrated_channels.utils import generate_formatted_log, stringify_and_store_api_record
 
-MOODLE_FINAL_GRADE_ASSIGNMENT_NAME = '(edX integration) Final Grade'
+LOGGER = logging.getLogger(__name__)
+
+
+class MoodleClientError(ClientError):
+    """
+    Indicate a problem when interacting with Moodle.
+    """
+
+    def __init__(self, message, status_code=500, moodle_error=None):
+        """Save the status code and message raised from the client."""
+        self.status_code = status_code
+        self.message = message
+        self.moodle_error = moodle_error
+        super().__init__(message, status_code)
+
+
+class MoodleResponse:
+    """
+    Represents an HTTP response with status code and textual content.
+    """
+
+    def __init__(self, status_code, text):
+        """Save the status code and text of the response."""
+        self.status_code = status_code
+        self.text = text
 
 
 def moodle_request_wrapper(method):
@@ -22,6 +48,7 @@ def moodle_request_wrapper(method):
     Wraps requests to Moodle's API in a token check.
     Will obtain a new token if there isn't one.
     """
+
     def inner(self, *args, **kwargs):
         if not self.token:
             self.token = self._get_access_token()  # pylint: disable=protected-access
@@ -33,9 +60,10 @@ def moodle_request_wrapper(method):
             # This cannot be converted to JSON thus the above fails miserably.
             # The above can fail with different errors depending on the format of the returned page.
             # Moodle of course does not tell us what is wrong in any part of this HTML.
-            raise ClientError('Moodle API task "{method}" failed due to unknown error.'.format(
-                method=method.__name__), response.status_code
-            ) from error
+            log_msg = (f'Moodle API task "{method.__name__}" '
+                       f'for enterprise_customer_uuid "{self.enterprise_configuration.enterprise_customer.uuid}" '
+                       f'failed due to unknown error with code "{response.status_code}".')
+            raise ClientError(log_msg, response.status_code) from error
         if isinstance(body, list):
             # On course creation (and ONLY course creation) success,
             # Moodle returns a list of JSON objects, because of course it does.
@@ -45,6 +73,8 @@ def moodle_request_wrapper(method):
             # This only happens for grades AFAICT. Zero also doesn't necessarily mean success,
             # but we have nothing else to go on
             if body == 0:
+                if method.__name__ == "_wrapped_create_course_completion" and response.status_code == 200:
+                    return MoodleResponse(status_code=200, text='')
                 return 200, ''
             raise ClientError('Moodle API Grade Update failed with int code: {code}'.format(code=body), 500)
         if isinstance(body, str):
@@ -56,12 +86,13 @@ def moodle_request_wrapper(method):
             self.token = self._get_access_token()  # pylint: disable=protected-access
             response = method(self, *args, **kwargs)
         elif error_code:
-            raise ClientError(
+            raise MoodleClientError(
                 'Moodle API Client Task "{method}" failed with error code '
                 '"{code}" and message: "{msg}" '.format(
                     method=method.__name__, code=error_code, msg=body.get('message'),
                 ),
-                response.status_code
+                response.status_code,
+                error_code,
             )
         elif warnings:
             # More Moodle nonsense!
@@ -92,7 +123,7 @@ class MoodleAPIClient(IntegratedChannelApiClient):
         - Customer's Moodle service short name
     """
 
-    MOODLE_API_PATH = '/webservice/rest/server.php'
+    MOODLE_API_PATH = 'webservice/rest/server.php'
 
     def __init__(self, enterprise_configuration):
         """
@@ -104,34 +135,53 @@ class MoodleAPIClient(IntegratedChannelApiClient):
         """
         super().__init__(enterprise_configuration)
         self.config = apps.get_app_config('moodle')
-        self.token = enterprise_configuration.token or self._get_access_token()
+        self.token = enterprise_configuration.decrypted_token or self._get_access_token()
+        self.api_url = urljoin(self.enterprise_configuration.moodle_base_url, self.MOODLE_API_PATH)
+        self.IntegratedChannelAPIRequestLogs = apps.get_model(
+            "integrated_channel", "IntegratedChannelAPIRequestLogs"
+        )
 
-    def _post(self, additional_params, method_url=None):
+    def _post(self, additional_params):
         """
         Compile common params and run request's post function
         """
+        headers = {
+            'Content-Type': 'application/x-www-form-urlencoded',
+        }
         params = {
             'wstoken': self.token,
             'moodlewsrestformat': 'json',
         }
         params.update(additional_params)
-        if method_url:
-            response = requests.post(
-                url='{url}&{querystring}'.format(
-                    url=method_url,
-                    querystring=urlencode(params)
-                )
-            )
-        else:
-            response = requests.post(
-                url='{url}{api_path}?{querystring}'.format(
-                    url=self.enterprise_configuration.moodle_base_url,
-                    api_path=self.MOODLE_API_PATH,
-                    querystring=urlencode(params)
-                )
-            )
+
+        start_time = time.time()
+        response = requests.post(
+            url=self.api_url,
+            data=params,
+            headers=headers
+        )
+        duration_seconds = time.time() - start_time
+        stringify_and_store_api_record(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=self.api_url,
+            data=params,
+            time_taken=duration_seconds,
+            status_code=response.status_code,
+            response_body=response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
+
         return response
 
+    @moodle_request_wrapper
+    def _wrapped_post(self, additional_params):
+        """
+        A version of _post which handles error cases, useful
+        for when the caller wants to examine errors
+        """
+        return self._post(additional_params)
+
     def _get_access_token(self):
         """
         Obtains a new access token from Moodle using username and password.
@@ -140,19 +190,31 @@ class MoodleAPIClient(IntegratedChannelApiClient):
             'service': self.enterprise_configuration.service_short_name
         }
 
+        url = urljoin(self.enterprise_configuration.moodle_base_url, 'login/token.php')
+        complete_url = "{}?{}".format(url, urlencode(querystring))
+        start_time = time.time()
+        data = {
+            "username": self.enterprise_configuration.decrypted_username,
+            "password": self.enterprise_configuration.decrypted_password,
+        }
         response = requests.post(
-            urljoin(
-                self.enterprise_configuration.moodle_base_url,
-                '/login/token.php',
-            ),
+            url,
             params=querystring,
             headers={
                 'Content-Type': 'application/x-www-form-urlencoded',
             },
-            data={
-                'username': self.enterprise_configuration.username,
-                'password': self.enterprise_configuration.password,
-            },
+            data=data,
+        )
+        duration_seconds = time.time() - start_time
+        stringify_and_store_api_record(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=complete_url,
+            data=data,
+            time_taken=duration_seconds,
+            status_code=response.status_code,
+            response_body=response.text,
+            channel_name=self.enterprise_configuration.channel_code()
         )
 
         try:
@@ -212,13 +274,23 @@ class MoodleAPIClient(IntegratedChannelApiClient):
             'courseid': course_id,
             'moodlewsrestformat': 'json'
         }
+        complete_url = "{}?{}".format(self.api_url, urlencode(params))
+        start_time = time.time()
         response = requests.get(
-            urljoin(
-                self.enterprise_configuration.moodle_base_url,
-                self.MOODLE_API_PATH,
-            ),
+            self.api_url,
             params=params
         )
+        duration_seconds = time.time() - start_time
+        self.IntegratedChannelAPIRequestLogs.store_api_call(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=complete_url,
+            payload='',
+            time_taken=duration_seconds,
+            status_code=response.status_code,
+            response_body=response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
         return response
 
     def get_course_final_grade_module(self, course_id):
@@ -231,14 +303,14 @@ class MoodleAPIClient(IntegratedChannelApiClient):
             - module_name (string): The string name of the module. Required for sending a grade update request.
         """
         response = self._get_course_contents(course_id)
-
         course_module_id = None
+        module_name = None
         if isinstance(response.json(), list):
             for course in response.json():
                 if course.get('name') == 'General':
                     modules = course.get('modules')
                     for module in modules:
-                        if module.get('name') == MOODLE_FINAL_GRADE_ASSIGNMENT_NAME:
+                        if module.get('name') == self.enterprise_configuration.grade_assignment_name:
                             course_module_id = module.get('id')
                             module_name = module.get('modname')
 
@@ -259,32 +331,52 @@ class MoodleAPIClient(IntegratedChannelApiClient):
         params = {
             'wstoken': self.token,
             'wsfunction': 'core_course_get_courses_by_field',
-            'field': 'shortname',
+            'field': 'idnumber',
             'value': key,
             'moodlewsrestformat': 'json'
         }
+        complete_url = "{}?{}".format(self.api_url, urlencode(params))
+        start_time = time.time()
         response = requests.get(
-            urljoin(
-                self.enterprise_configuration.moodle_base_url,
-                self.MOODLE_API_PATH,
-            ),
+            self.api_url,
             params=params
         )
+        duration_seconds = time.time() - start_time
+        self.IntegratedChannelAPIRequestLogs.store_api_call(
+            enterprise_customer=self.enterprise_configuration.enterprise_customer,
+            enterprise_customer_configuration_id=self.enterprise_configuration.id,
+            endpoint=complete_url,
+            payload='',
+            time_taken=duration_seconds,
+            status_code=response.status_code,
+            response_body=response.text,
+            channel_name=self.enterprise_configuration.channel_code()
+        )
         return response
 
-    def get_course_id(self, key):
+    def _get_course_id(self, key):
         """
-        Obtain course from Moodle by course key and parse out the id.
+        Obtain course from Moodle by course key and parse out the id. No exception thrown.
         """
         response = self._get_courses(key)
         parsed_response = json.loads(response.text)
         if not parsed_response.get('courses'):
+            return None
+        else:
+            return parsed_response['courses'][0]['id']
+
+    def get_course_id(self, key):
+        """
+        Obtain course from Moodle by course key and parse out the id. Raise on not found.
+        """
+        course_id = self._get_course_id(key)
+        if not course_id:
             raise ClientError(
                 'MoodleAPIClient request failed: 404 Course key '
                 '"{}" not found in Moodle.'.format(key),
                 HTTPStatus.NOT_FOUND.value
             )
-        return parsed_response['courses'][0]['id']
+        return course_id
 
     @moodle_request_wrapper
     def _wrapped_create_course_completion(self, user_id, payload):
@@ -307,11 +399,41 @@ class MoodleAPIClient(IntegratedChannelApiClient):
             'itemnumber': 0,
             'grades[0][studentid]': moodle_user_id,
             # The grade is exported as a decimal between [0-1]
-            'grades[0][grade]': completion_data['grade'] * 100
+            'grades[0][grade]': completion_data['grade'] * self.enterprise_configuration.grade_scale
         }
-        return self._post(params)
 
-    @moodle_request_wrapper
+        response = self._post(params)
+
+        if hasattr(response, 'status_code'):
+            status_code = response.status_code
+        else:
+            status_code = None
+
+        if hasattr(response, 'text'):
+            text = response.text
+        else:
+            text = None
+
+        if hasattr(response, 'headers'):
+            headers = response.headers
+        else:
+            headers = None
+
+        LOGGER.info(
+            'Learner Data Transmission'
+            f'for course={completion_data["courseID"]}  with data '
+            f'source: {module_name}, '
+            f'activityid: {course_module_id}, '
+            f'grades[0][studentid]: {moodle_user_id}, '
+            f'grades[0][grade]: {completion_data["grade"] * self.enterprise_configuration.grade_scale} '
+            f' with response: {response} '
+            f'Status Code: {status_code}, '
+            f'Text: {text}, '
+            f'Headers: {headers}, '
+        )
+
+        return response
+
     def create_content_metadata(self, serialized_data):
         """
         The below assumes the data is dict/object.
@@ -324,39 +446,103 @@ class MoodleAPIClient(IntegratedChannelApiClient):
           courses[1][fullname]: 'value',
           [...]
         }
+        when sending 1 course and its a dupe, treat as success.
+        when sending N courses and a dupe exists, throw exception since
+        there is no easy way to retry with just the non-dupes.
         """
+        # check to see if more than 1 course is being passed
+        more_than_one_course = serialized_data.get('courses[1][shortname]')
         serialized_data['wsfunction'] = 'core_course_create_courses'
-        response = self._post(serialized_data)
-        return response
+        try:
+            moodle_course_id = self._get_course_id(serialized_data['courses[0][idnumber]'])
+            # Course already exists but is hidden - make it visible
+            if moodle_course_id:
+                serialized_data['courses[0][visible]'] = 1
+                return self.update_content_metadata(serialized_data)
+            else:  # create a new course
+                response = self._wrapped_post(serialized_data)
+        except MoodleClientError as error:
+            # treat duplicate as successful, but only if its a single course
+            # set chunk size settings to 1 if youre seeing a lot of these errors
+            if error.moodle_error == 'shortnametaken' and not more_than_one_course:
+                return 200, "shortnametaken"
+            elif error.moodle_error == 'courseidnumbertaken' and not more_than_one_course:
+                return 200, "courseidnumbertaken"
+            else:
+                raise error
+        return response.status_code, response.text
 
-    @moodle_request_wrapper
     def update_content_metadata(self, serialized_data):
-        moodle_course_id = self.get_course_id(serialized_data['courses[0][shortname]'])
-        serialized_data['courses[0][id]'] = moodle_course_id
-        serialized_data['wsfunction'] = 'core_course_update_courses'
-        return self._post(serialized_data)
+        moodle_course_id = self._get_course_id(serialized_data['courses[0][idnumber]'])
+        # if we cannot find the course, lets create it
+        if moodle_course_id:
+            serialized_data['courses[0][id]'] = moodle_course_id
+            serialized_data['wsfunction'] = 'core_course_update_courses'
+            response = self._wrapped_post(serialized_data)
+            return response.status_code, response.text
+        else:
+            return self.create_content_metadata(serialized_data)
 
-    @moodle_request_wrapper
     def delete_content_metadata(self, serialized_data):
-        moodle_course_id = self.get_course_id(serialized_data['courses[0][shortname]'])
-        params = {
-            'wsfunction': 'core_course_delete_courses',
-            'courseids[]': moodle_course_id
-        }
-        return self._post(params)
+        response = self._get_courses(serialized_data['courses[0][idnumber]'])
+        parsed_response = json.loads(response.text)
+        if not parsed_response.get('courses'):
+            LOGGER.info(
+                generate_formatted_log(
+                    self.enterprise_configuration.channel_code(),
+                    self.enterprise_configuration.enterprise_customer.uuid,
+                    None,
+                    None,
+                    'No course found while attempting to delete edX course: '
+                    f'{serialized_data["courses[0][idnumber]"]} from moodle.'
+                )
+            )
+            # Hacky way of getting around the request wrapper validation
+            rsp = requests.Response()
+            rsp._content = bytearray('{"result": "Course not found."}', 'utf-8')  # pylint: disable=protected-access
+            return rsp
+        moodle_course_id = parsed_response['courses'][0]['id']
+        serialized_data['wsfunction'] = 'core_course_update_courses'
+        serialized_data['courses[0][id]'] = moodle_course_id
+        response = self._wrapped_post(serialized_data)
+        return response.status_code, response.text
 
     def create_assessment_reporting(self, user_id, payload):
         """
         Not implemented yet
         """
 
+    def cleanup_duplicate_assignment_records(self, courses):
+        """
+        Not implemented yet.
+        """
+        LOGGER.error(
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                None,
+                None,
+                "Moodle integrated channel does not yet support assignment deduplication."
+            )
+        )
+
     def create_course_completion(self, user_id, payload):
         """Send course completion data to Moodle"""
         # The base integrated channels transmitter expects a tuple of (code, body),
         # but we need to wrap the requests
         resp = self._wrapped_create_course_completion(user_id, payload)
+        completion_data = json.loads(payload)
+        LOGGER.info(
+            generate_formatted_log(
+                channel_name=self.enterprise_configuration.channel_code(),
+                enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
+                course_or_course_run_key=completion_data['courseID'],
+                plugin_configuration_id=self.enterprise_configuration.id,
+                message=f'Response for Moodle Create Course Completion Request response: {resp} '
+            )
+        )
         return resp.status_code, resp.text
 
     @moodle_request_wrapper
-    def delete_course_completion(self, user_id, payload):  # pylint: disable=unused-argument
+    def delete_course_completion(self, user_id, payload):
         pass
