@@ -1,14 +1,21 @@
-# -*- coding: utf-8 -*-
 """
 Content metadata exporter for SAP SuccessFactors.
 """
 
 from logging import getLogger
 
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
-from enterprise.api_client.lms import parse_lms_api_datetime
-from enterprise.utils import get_closest_course_run, is_course_run_available_for_enrollment
+from enterprise.constants import IC_DELETE_ACTION
+from enterprise.utils import (
+    get_advertised_course_run,
+    get_advertised_or_closest_course_run,
+    get_closest_course_run,
+    get_duration_of_course_or_courserun,
+    is_course_run_active,
+    is_course_run_available_for_enrollment,
+    parse_lms_api_datetime,
+)
 from enterprise.views import CourseEnrollmentView
 from integrated_channels.integrated_channel.exporters.content_metadata import ContentMetadataExporter
 from integrated_channels.sap_success_factors.exporters.utils import transform_language_code
@@ -23,7 +30,7 @@ from integrated_channels.utils import (
 LOGGER = getLogger(__name__)
 
 
-class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pylint: disable=abstract-method
+class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):
     """
     SAP SuccessFactors implementation of ContentMetadataExporter.
     """
@@ -41,17 +48,36 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
         'price': 'price',
     }
 
+    def _apply_delete_transformation(self, metadata):
+        """
+        Specific transformations required for "deleting" a course on a SAP external service.
+        """
+        # Applying the metadata payload update to "delete" the course on SAP instances
+        metadata['status'] = 'INACTIVE'
+
+        # Sanity check as we've seen issues with schedule structure
+        metadata_schedule = metadata.get('schedule')
+        if metadata_schedule:
+            schedule = metadata_schedule[0]
+            if not schedule.get('startDate') or not schedule.get('endDate'):
+                metadata['schedule'] = []
+        return metadata
+
     def transform_provider_id(self, content_metadata_item):  # pylint: disable=unused-argument
         """
         Return the provider ID from the integrated channel configuration.
         """
         return self.enterprise_configuration.provider_id
 
-    def transform_status(self, content_metadata_item):  # pylint: disable=unused-argument
+    def transform_for_action_status(self, _content_metadata_item, action):
         """
         Return the status of the content item.
         """
-        return 'ACTIVE'
+        # lets not overwrite something we've already tried to set INACTIVE
+        if action == IC_DELETE_ACTION:
+            return 'INACTIVE'
+        else:
+            return 'ACTIVE'
 
     def transform_title(self, content_metadata_item):
         """
@@ -79,17 +105,21 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
             content_metadata_item.get('title', '')
         )
 
-        course_runs = content_metadata_item.get('course_runs')
-        if course_runs:
-            closest_course_run = get_closest_course_run(course_runs)
+        course_run = get_advertised_course_run(content_metadata_item)
 
+        if not course_run:
+            course_runs = content_metadata_item.get('course_runs')
+            if course_runs:
+                course_run = get_closest_course_run(course_runs)
+
+        if course_run:
             # Include the course run start and end dates
-            date_str = self._get_course_run_start_end_str(closest_course_run)
+            date_str = self._get_course_run_start_end_str(course_run)
             if date_str:
                 description = date_str + description
 
             # Include the course pacing
-            course_pacing = CourseEnrollmentView.PACING_FORMAT.get(closest_course_run['pacing_type'], '')
+            course_pacing = CourseEnrollmentView.PACING_FORMAT.get(course_run['pacing_type'], '')
             if course_pacing:
                 pacing_desc = 'Pacing: {pacing_type}. '.format(
                     pacing_type=course_pacing
@@ -123,7 +153,7 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
             'providerID': self.enterprise_configuration.provider_id,
             'launchURL': content_metadata_item['enrollment_url'],
             'contentTitle': content_metadata_item['title'],
-            'contentID': self.get_content_id(content_metadata_item),
+            'contentID': self._get_content_id(content_metadata_item),
             'launchType': 3,  # This tells SAPSF to launch the course in a new browser window.
             'mobileEnabled': True,  # Always return True per ENT-1401
             'mobileLaunchURL': content_metadata_item['enrollment_url'],
@@ -135,21 +165,43 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
         """
         return 1
 
-    def transform_schedule(self, content_metadata_item):  # pylint: disable=unused-argument
+    def transform_schedule(self, content_metadata_item):
         """
         Return the schedule of the content item.
         """
-        return []
+        duration, start, end = get_duration_of_course_or_courserun(content_metadata_item)
+        if content_metadata_item.get('content_type') == 'course':
+            course_run = get_advertised_or_closest_course_run(content_metadata_item)
+            if course_run:
+                content_metadata_item = course_run
+
+        # SAP will throw errors if we try to send an empty start or end date
+        if (not start or not end):
+            return []
+
+        return [{
+            'startDate': parse_datetime_to_epoch_millis(start) if start else '',
+            'endDate': parse_datetime_to_epoch_millis(end) if end else '',
+            'active': is_course_run_available_for_enrollment(content_metadata_item),
+            'duration': f"{duration} days",
+        }]
 
     def transform_price(self, content_metadata_item):
         """
         Return the current course run's price.
         """
         price = 0.0
+
         if self.enterprise_configuration.show_course_price:
-            for course_run in content_metadata_item.get('course_runs', []):
-                if course_run['availability'] == 'Current':
-                    price = course_run.get('first_enrollable_paid_seat_price', 0.0) or 0.0
+            advertised_course_run = get_advertised_course_run(content_metadata_item)
+            if advertised_course_run and 'first_enrollable_paid_seat_price' in advertised_course_run:
+                price = advertised_course_run.get('first_enrollable_paid_seat_price') or 0.0
+            else:
+                for course_run in content_metadata_item.get('course_runs', []):
+                    if 'first_enrollable_paid_seat_price' in course_run and is_course_run_active(course_run):
+                        price = course_run.get('first_enrollable_paid_seat_price') or 0.0
+                        break
+
         return [
             {
                 "currency": "USD",
@@ -186,37 +238,6 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
 
         return title_with_locales
 
-    def _get_course_run_start_end_str(self, course_run):
-        """
-        Get the course run start and end as a descriptive string. Also include a note if enrollment is closed.
-        """
-        course_run_start = course_run.get('start')
-        course_run_end = course_run.get('end')
-        date_str = ''
-
-        if course_run_start:
-            date_str += '{starts}: {:%B %Y}'.format(
-                parse_lms_api_datetime(course_run_start),
-                starts=_('Starts')
-            )
-
-        if course_run_end:
-            if date_str:
-                date_str += ', '
-
-            date_str += '{ends}: {:%B %Y}. '.format(
-                parse_lms_api_datetime(course_run_end),
-                ends=_('Ends')
-            )
-        else:
-            if date_str:
-                date_str += '. '
-
-        if not is_course_run_available_for_enrollment(course_run):
-            date_str += 'Enrollment is closed. '
-
-        return date_str
-
     def transform_courserun_description(self, content_metadata_item):
         """
         Return the description of the courserun content item.
@@ -254,7 +275,7 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
         """
         return content_metadata_item['uuid']
 
-    def get_content_id(self, content_metadata_item):
+    def _get_content_id(self, content_metadata_item):
         """
         Return the id for the given content_metadata_item, `uuid` for programs or `key` for other content
         """
@@ -262,3 +283,34 @@ class SapSuccessFactorsContentMetadataExporter(ContentMetadataExporter):  # pyli
         if content_metadata_item['content_type'] == 'program':
             content_id = content_metadata_item.get('uuid', '')
         return content_id
+
+    def _get_course_run_start_end_str(self, course_run):
+        """
+        Get the course run start and end as a descriptive string. Also include a note if enrollment is closed.
+        """
+        course_run_start = course_run.get('start')
+        course_run_end = course_run.get('end')
+        date_str = ''
+
+        if course_run_start:
+            date_str += '{starts}: {:%B %Y}'.format(
+                parse_lms_api_datetime(course_run_start),
+                starts=_('Starts')
+            )
+
+        if course_run_end:
+            if date_str:
+                date_str += ', '
+
+            date_str += '{ends}: {:%B %Y}. '.format(
+                parse_lms_api_datetime(course_run_end),
+                ends=_('Ends')
+            )
+        else:
+            if date_str:
+                date_str += '. '
+
+        if not is_course_run_available_for_enrollment(course_run):
+            date_str += 'Enrollment is closed. '
+
+        return date_str
