@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Learner data exporter for Enterprise Integrated Channel SAP SuccessFactors.
 """
@@ -9,13 +8,13 @@ from requests import RequestException
 
 from django.apps import apps
 
-from enterprise.api_client.discovery import get_course_catalog_api_service_client
 from enterprise.models import EnterpriseCustomerUser, PendingEnterpriseCustomerUser
 from enterprise.tpa_pipeline import get_user_from_social_auth
+from integrated_channels.catalog_service_utils import get_course_id_for_enrollment, get_course_run_for_enrollment
 from integrated_channels.exceptions import ClientError
 from integrated_channels.integrated_channel.exporters.learner_data import LearnerExporter
 from integrated_channels.sap_success_factors.client import SAPSuccessFactorsAPIClient
-from integrated_channels.utils import parse_datetime_to_epoch_millis
+from integrated_channels.utils import generate_formatted_log, parse_datetime_to_epoch_millis
 
 LOGGER = getLogger(__name__)
 
@@ -25,69 +24,76 @@ class SapSuccessFactorsLearnerExporter(LearnerExporter):
     Class to provide a SAPSF learner data transmission audit prepared for serialization.
     """
 
+    INCLUDE_GRADE_FOR_COMPLETION_AUDIT_CHECK = False
+
     def get_learner_data_records(
             self,
             enterprise_enrollment,
             completed_date=None,
             grade=None,
-            is_passing=False,
-            grade_percent=None
-    ):
+            content_title=None,
+            progress_status=None,
+            course_completed=False,
+            **kwargs,
+    ):   # pylint: disable=arguments-differ
         """
         Return a SapSuccessFactorsLearnerDataTransmissionAudit with the given enrollment and course completion data.
 
-        If completed_date is None and the learner isn't passing, then course completion has not been met.
-
         If no remote ID can be found, return None.
         """
-        completed_timestamp = None
-        course_completed = False
+        sap_completed_timestamp = None
         if completed_date is not None:
-            completed_timestamp = parse_datetime_to_epoch_millis(completed_date)
-            course_completed = is_passing
+            sap_completed_timestamp = parse_datetime_to_epoch_millis(completed_date)
 
-        sapsf_user_id = enterprise_enrollment.enterprise_customer_user.get_remote_id()
+        sapsf_user_id = enterprise_enrollment.enterprise_customer_user.get_remote_id(
+            self.enterprise_configuration.idp_id
+        )
 
         if sapsf_user_id is not None:
-            SapSuccessFactorsLearnerDataTransmissionAudit = apps.get_model(  # pylint: disable=invalid-name
+            SapSuccessFactorsLearnerDataTransmissionAudit = apps.get_model(
                 'sap_success_factors',
                 'SapSuccessFactorsLearnerDataTransmissionAudit'
             )
             # We return two records here, one with the course key and one with the course run id, to account for
             # uncertainty about the type of content (course vs. course run) that was sent to the integrated channel.
-            course_catalog_client = get_course_catalog_api_service_client(
-                site=enterprise_enrollment.enterprise_customer_user.enterprise_customer.site
-            )
-            course_run = course_catalog_client.get_course_run(enterprise_enrollment.course_id)
+            course_run = get_course_run_for_enrollment(enterprise_enrollment)
             total_hours = 0.0
             if course_run and self.enterprise_configuration.transmit_total_hours:
                 total_hours = course_run.get("estimated_hours", 0.0)
-            return [
-                SapSuccessFactorsLearnerDataTransmissionAudit(
+            course_id = get_course_id_for_enrollment(enterprise_enrollment)
+            # We only want to send one record per enrollment and course, so we check if one exists first.
+            learner_transmission_record = SapSuccessFactorsLearnerDataTransmissionAudit.objects.filter(
+                enterprise_course_enrollment_id=enterprise_enrollment.id,
+                course_id=course_id,
+            ).first()
+            if learner_transmission_record is None:
+                learner_transmission_record = SapSuccessFactorsLearnerDataTransmissionAudit(
                     enterprise_course_enrollment_id=enterprise_enrollment.id,
                     sapsf_user_id=sapsf_user_id,
-                    course_id=course_catalog_client.get_course_id(enterprise_enrollment.course_id),
+                    user_email=enterprise_enrollment.enterprise_customer_user.user_email,
+                    course_id=get_course_id_for_enrollment(enterprise_enrollment),
                     course_completed=course_completed,
-                    completed_timestamp=completed_timestamp,
+                    completed_timestamp=completed_date,
+                    sap_completed_timestamp=sap_completed_timestamp,
                     grade=grade,
+                    content_title=content_title,
+                    progress_status=progress_status,
                     total_hours=total_hours,
                     credit_hours=total_hours,
-                ),
-                SapSuccessFactorsLearnerDataTransmissionAudit(
-                    enterprise_course_enrollment_id=enterprise_enrollment.id,
-                    sapsf_user_id=sapsf_user_id,
-                    course_id=enterprise_enrollment.course_id,
-                    course_completed=course_completed,
-                    completed_timestamp=completed_timestamp,
-                    grade=grade,
-                    total_hours=total_hours,
-                    credit_hours=total_hours,
-                ),
-            ]
+                    enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
+                    plugin_configuration_id=self.enterprise_configuration.id
+                )
+            return [learner_transmission_record]
         LOGGER.info(
-            '[Integrated Channel] No learner data was sent for user [%s] because an SAP SuccessFactors user ID'
-            ' could not be found.',
-            enterprise_enrollment.enterprise_customer_user.username
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                enterprise_enrollment.enterprise_customer_user.user_id,
+                enterprise_enrollment.course_id,
+                '[Integrated Channel] No learner data was sent for user '
+                f'{enterprise_enrollment.enterprise_customer_user.username} because an SAP SuccessFactors user ID '
+                ' could not be found.'
+            )
         )
         return None
 
@@ -108,14 +114,8 @@ class SapSuccessFactorsLearnerManger:
         self.enterprise_configuration = enterprise_configuration
         self.client = client(enterprise_configuration) if client else None
 
-    def unlink_learners(self):
-        """
-        Iterate over each learner and unlink inactive SAP channel learners.
-
-        This method iterates over each enterprise learner and unlink learner
-        from the enterprise if the learner is marked inactive in the related
-        integrated channel.
-        """
+    def _get_inactive_learners(self):
+        """ Gets inactive learners list from the client or raises ClientError on failure. """
         try:
             sap_inactive_learners = self.client.get_inactive_sap_learners()
         except RequestException as exc:
@@ -125,32 +125,58 @@ class SapSuccessFactorsLearnerManger:
                     message=str(exc)
                 )
             ) from exc
+        return sap_inactive_learners
+
+    def _get_identity_providers(self):
+        """ Logic check for getting an identity provider preflight validation, split out for unit testing."""
+        enterprise_customer = self.enterprise_configuration.enterprise_customer
+        providers = enterprise_customer.identity_providers
+        if not enterprise_customer.has_identity_providers:
+            LOGGER.info(
+                generate_formatted_log(
+                    self.enterprise_configuration.channel_code(),
+                    self.enterprise_configuration.enterprise_customer.uuid,
+                    None,
+                    None,
+                    f'Enterprise customer {enterprise_customer.name} has no associated identity provider'
+                )
+            )
+            return None
+        return providers
+
+    def unlink_learners(self):
+        """
+        Iterate over each learner and unlink inactive SAP channel learners.
+
+        This method iterates over each enterprise learner and unlink learner
+        from the enterprise if the learner is marked inactive in the related
+        integrated channel.
+        """
+        sap_inactive_learners = self._get_inactive_learners()
+
         total_sap_inactive_learners = len(sap_inactive_learners) if sap_inactive_learners else 0
         enterprise_customer = self.enterprise_configuration.enterprise_customer
         LOGGER.info(
-            'Found [%d] SAP inactive learners for enterprise customer [%s]',
-            total_sap_inactive_learners, enterprise_customer.name
+            generate_formatted_log(
+                self.enterprise_configuration.channel_code(),
+                self.enterprise_configuration.enterprise_customer.uuid,
+                None,
+                None,
+                f'Found {total_sap_inactive_learners} SAP inactive learners for '
+                f'enterprise customer {enterprise_customer.name}'
+            )
         )
         if not sap_inactive_learners:
             return None
 
-        providers = enterprise_customer.identity_providers
-        if not enterprise_customer.has_identity_providers:
-            LOGGER.info(
-                'Enterprise customer [%s] has no associated identity provider',
-                enterprise_customer.name
-            )
+        providers = self._get_identity_providers()
+        if not providers:
             return None
 
         for sap_inactive_learner in sap_inactive_learners:
             sap_student_id = sap_inactive_learner['studentID']
-            social_auth_user = get_user_from_social_auth(providers, sap_student_id)
+            social_auth_user = get_user_from_social_auth(providers, sap_student_id, enterprise_customer)
             if not social_auth_user:
-                LOGGER.info(
-                    'No social auth data found for inactive user with SAP student id [%s] of enterprise '
-                    'customer [%s] with identity providers [%s]',
-                    sap_student_id, enterprise_customer.name, ', '.join(provider.provider_id for provider in providers)
-                )
                 continue
 
             try:
@@ -161,9 +187,13 @@ class SapSuccessFactorsLearnerManger:
                 )
             except (EnterpriseCustomerUser.DoesNotExist, PendingEnterpriseCustomerUser.DoesNotExist):
                 LOGGER.info(
-                    'Learner with email [%s] and SAP student id [%s] is not linked with enterprise [%s]',
-                    social_auth_user.email,
-                    sap_student_id,
-                    enterprise_customer.name
+                    generate_formatted_log(
+                        self.enterprise_configuration.channel_code(),
+                        self.enterprise_configuration.enterprise_customer.uuid,
+                        None,
+                        None,
+                        f'Learner with email {social_auth_user.email} and SAP student id {sap_student_id} '
+                        f'is not linked with enterprise {enterprise_customer.name}'
+                    )
                 )
         return None
