@@ -1,12 +1,10 @@
-# -*- coding: utf-8 -*-
-
 """
 xAPI Client to send payload data.
 """
 
 import logging
 
-from tincan.remote_lrs import RemoteLRS  # pylint: disable=E0401
+from tincan.remote_lrs import RemoteLRS
 
 from integrated_channels.exceptions import ClientError
 
